/* $AstecId: Xrt3d_xtype.c,v 1.5 2000/06/19 08:32:08 kosaka Exp $ */
/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/3d widget UIM/X integration file
 *
 *  SCCSID -- @(#)Xrt3d_xtype.c	1.16    99/02/01 KL Group Inc.
 */

/******************************************************************************
				Xrt3d_xtype.c
******************************************************************************/

#include <Xm/Xm.h>

#include <UxLib.h>
#include <vtypes.h>
#include <utype.h>
#include <ctype.h>
#include <string.h>

#include <malloc.h>

#ifdef DESIGN_TIME

#include "veos.h"       		/* Visual Edge object system */
#include "valuesOf.h"   		/* ValuesOf & Validator functions */
#include "resource.h"
#include "resvalue.h"
#include "plist.cl.h"
#include "observer.h"

#endif /* DESIGN_TIME */

#include "Xrt_restable.h"

#include   <Xm/Xrt3d.h>

#include <cat_macros.h>

/* Begin new XRT/3d types */

int     UxXT_Xrt3dAdjust;
int     UxXT_Xrt3dAlign;
int     UxXT_Xrt3dAnchor;
int     UxXT_Xrt3dAnnoMethod;
int     UxXT_Xrt3dAttachMethod;
int     UxXT_Xrt3dBarFormat;
int     UxXT_Xrt3dBorder;
int     UxXT_Xrt3dContourStyles;
int     UxXT_Xrt3dDistnMethod;
int     UxXT_Xrt3dDistnTable;
int     UxXT_Xrt3dDouble;
int     UxXT_Xrt3dLegendStyle;
int     UxXT_Xrt3dPreviewMethod;
int     UxXT_Xrt3dPlane;
int     UxXT_Xrt3dProject;
int     UxXT_Xrt3dStrings;
int     UxXT_Xrt3dStrokeFont;
int     UxXT_Xrt3dSurfaceData;
int     UxXT_Xrt3dTextList;
int     UxXT_Xrt3dType;
int     UxXT_Xrt3dValueLabels;
int     UxXT_Xrt3dXYColors;
int     UxXT_Xrt3dXGridLines;
int     UxXT_Xrt3dYGridLines;
int     UxXT_Xrt3dZGridLines;
int     UxXT_Xrt3dZoneData;
int     UxXT_Xrt3dZoneMethod;

int     UxXT_Xrt3dDataStyles;
int     UxXT_Xrt3dDistnRange;
int     UxXT_Xrt3dImage;
int     UxXT_Xrt3dImageLayout;
int     UxXT_Xrt3dLineStyle;

/* Define three static arrays for each enumerated type */

/* Adjust */
static char * uXrt3dAdjust[] = {
	"AdjustCenter",
	"AdjustRight",
	"AdjustLeft"
};
static int xXrt3dAdjust[] = {
	XRT3D_ADJUST_CENTER,
	XRT3D_ADJUST_RIGHT,
	XRT3D_ADJUST_LEFT
};
static char * dXrt3dAdjust[] = {
	"XRT3D_ADJUST_CENTER",
	"XRT3D_ADJUST_RIGHT",
	"XRT3D_ADJUST_LEFT"
};

/* Align */
static char * uXrt3dAlign[] = {
	"AlignVertical",
	"AlignHorizontal"
};
static int xXrt3dAlign[] = {
	XRT3D_ALIGN_VERTICAL,
	XRT3D_ALIGN_HORIZONTAL
};
static char * dXrt3dAlign[] = {
	"XRT3D_ALIGN_VERTICAL",
	"XRT3D_ALIGN_HORIZONTAL"
};

/* Anchors */
static char * uXrt3dAnchor[] = {
	"AnchorWest",
	"AnchorEast",
	"AnchorSouth",
	"AnchorNorth",
	"AnchorSoutheast",
	"AnchorSouthwest",
	"AnchorNortheast",
	"AnchorNorthwest"
};
static int xXrt3dAnchor[] = {
	XRT3D_ANCHOR_WEST,
	XRT3D_ANCHOR_EAST,
	XRT3D_ANCHOR_SOUTH,
	XRT3D_ANCHOR_NORTH,
	XRT3D_ANCHOR_SOUTHEAST,
	XRT3D_ANCHOR_SOUTHWEST,
	XRT3D_ANCHOR_NORTHEAST,
	XRT3D_ANCHOR_NORTHWEST
};
static char * dXrt3dAnchor[] = {
	"XRT3D_ANCHOR_WEST",
	"XRT3D_ANCHOR_EAST",
	"XRT3D_ANCHOR_SOUTH",
	"XRT3D_ANCHOR_NORTH",
	"XRT3D_ANCHOR_SOUTHEAST",
	"XRT3D_ANCHOR_SOUTHWEST",
	"XRT3D_ANCHOR_NORTHEAST",
	"XRT3D_ANCHOR_NORTHWEST"
};

/* Annotation Method */
static char * uXrt3dAnnoMethod[] = {
	"AnnoValues",
	"AnnoDataLabels",
	"AnnoValuesLabels"
};
static int xXrt3dAnnoMethod[] = {
	XRT3D_ANNO_VALUES,
	XRT3D_ANNO_DATA_LABELS,
	XRT3D_ANNO_VALUE_LABELS
};
static char * dXrt3dAnnoMethod[] = {
	"XRT3D_ANNO_VALUES",
	"XRT3D_ANNO_DATA_LABELS",
	"XRT3D_ANNO_VALUE_LABELS"
};

/* Attach Method */
static char * uXrt3dAttachMethod[] = {
	"AttachIndex",
	"AttachPixel",
	"AttachPoint"
};
static int xXrt3dAttachMethod[] = {
	XRT3D_ATTACH_INDEX,
	XRT3D_ATTACH_PIXEL,
	XRT3D_ATTACH_POINT
};
static char * dXrt3dAttachMethod[] = {
	"XRT3D_ATTACH_INDEX",
	"XRT3D_ATTACH_PIXEL",
	"XRT3D_ATTACH_POINT"
};

/* Bar Format */
static char * uXrt3dBarFormat[] = {
	"BarFixed",
	"BarHistogram"
};
static int xXrt3dBarFormat[] = {
	XRT3D_BAR_FIXED,
	XRT3D_BAR_HISTOGRAM
};
static char * dXrt3dBarFormat[] = {
	"XRT3D_BAR_FIXED",
	"XRT3D_BAR_HISTOGRAM"
};

/* Border Styles */
static char * uXrt3dBorder[] = {
	"BorderNone",
	"Border3dIn",
	"Border3dOut",
	"BorderShadow",
	"BorderPlain",
	"BorderEtchedIn",
	"BorderEtchedOut",
	"BorderFrameIn",
	"BorderFrameOut",
	"BorderBevel"
};
static int xXrt3dBorder[] = {
	XRT3D_BORDER_NONE,
	XRT3D_BORDER_3D_IN,
	XRT3D_BORDER_3D_OUT,
	XRT3D_BORDER_SHADOW,
	XRT3D_BORDER_PLAIN,
	XRT3D_BORDER_ETCHED_IN,
	XRT3D_BORDER_ETCHED_OUT,
	XRT3D_BORDER_FRAME_IN,
	XRT3D_BORDER_FRAME_OUT,
	XRT3D_BORDER_BEVEL
};
static char * dXrt3dBorder[] = {
	"XRT3D_BORDER_NONE",
	"XRT3D_BORDER_3D_IN",
	"XRT3D_BORDER_3D_OUT",
	"XRT3D_BORDER_SHADOW",
	"XRT3D_BORDER_PLAIN",
	"XRT3D_BORDER_ETCHED_IN",
	"XRT3D_BORDER_ETCHED_OUT",
	"XRT3D_BORDER_FRAME_IN",
	"XRT3D_BORDER_FRAME_OUT",
	"XRT3D_BORDER_BEVEL"
};

/* Distribution Methods */
static char * uXrt3dDistnMethod[] = {
	"DistnLinear",
	"DistnTable"
};
static int xXrt3dDistnMethod[] = {
	XRT3D_DISTN_LINEAR,
	XRT3D_DISTN_TABLE
};
static char * dXrt3dDistnMethod[] = {
	"XRT3D_DISTN_LINEAR",
	"XRT3D_DISTN_TABLE"
};

/* Legend Styles */
static char * uXrt3dLegendStyle[] = {
	"LegendStyleContinuous",
	"LegendStyleStepped"
};
static int xXrt3dLegendStyle[] = {
	XRT3D_LEGEND_STYLE_CONTINUOUS,
	XRT3D_LEGEND_STYLE_STEPPED
};
static char * dXrt3dLegendStyle[] = {
	"XRT3D_LEGEND_STYLE_CONTINUOUS",
	"XRT3D_LEGEND_STYLE_STEPPED"
};

/* Preview Method */
static char * uXrt3dPreviewMethod[] = {
	"PreviewCube",
	"PreviewFull"
};
static int xXrt3dPreviewMethod[] = {
	XRT3D_PREVIEW_CUBE,
	XRT3D_PREVIEW_FULL
};
static char * dXrt3dPreviewMethod[] = {
	"XRT3D_PREVIEW_CUBE",
	"XRT3D_PREVIEW_FULL"
};

/* Plane */
static char * uXrt3dPlane[] = {
	"XZPlane",
	"YZPlane"
	};
static int xXrt3dPlane[] = {
	XRT3D_XZ_PLANE,
	XRT3D_YZ_PLANE
	};
static char * dXrt3dPlane[] = {
	"XRT3D_XZ_PLANE",
	"XRT3D_YZ_PLANE"
	};

/* Project */
static char * uXrt3dProject[] = {
	"None",
	"ProjectContours",
	"ProjectZones",
	"ProjectContours + ProjectZones"
	};
static int xXrt3dProject[] = {
	0,
	XRT3D_PROJECT_CONTOURS,
	XRT3D_PROJECT_ZONES,
	XRT3D_PROJECT_CONTOURS + XRT3D_PROJECT_ZONES
	};
static char * dXrt3dProject[] = {
	"0",
	"XRT3D_PROJECT_CONTOURS",
	"XRT3D_PROJECT_ZONES",
	"XRT3D_PROJECT_CONTOURS + XRT3D_PROJECT_ZONES"
	};

/* Stroke Font */
static char * uXrt3dStrokeFont[] = {
	"SfRomanTriplex",
	"SfItalicComplex",
	"SfItalicComplexSmall",
	"SfItalicTriplex",
	"SfRomanComplex",
	"SfRomanComplexSmall",
	"SfRomanDuplex",
	"SfRomanSimplex"
};
static int xXrt3dStrokeFont[] = {
	XRT3D_SF_ROMAN_TRIPLEX,
	XRT3D_SF_ITALIC_COMPLEX,
	XRT3D_SF_ITALIC_COMPLEX_SMALL,
	XRT3D_SF_ITALIC_TRIPLEX,
	XRT3D_SF_ROMAN_COMPLEX,
	XRT3D_SF_ROMAN_COMPLEX_SMALL,
	XRT3D_SF_ROMAN_DUPLEX,
	XRT3D_SF_ROMAN_SIMPLEX
};
static char * dXrt3dStrokeFont[] = {
	"XRT3D_SF_ROMAN_TRIPLEX",
	"XRT3D_SF_ITALIC_COMPLEX",
	"XRT3D_SF_ITALIC_COMPLEX_SMALL",
	"XRT3D_SF_ITALIC_TRIPLEX",
	"XRT3D_SF_ROMAN_COMPLEX",
	"XRT3D_SF_ROMAN_COMPLEX_SMALL",
	"XRT3D_SF_ROMAN_DUPLEX",
	"XRT3D_SF_ROMAN_SIMPLEX"
};

/* Type of Surface */
static char * uXrt3dType[] = {
	"TypeSurface",
	"TypeBar",
	"TypeScatter"
};
static int xXrt3dType[] = {
	XRT3D_TYPE_SURFACE,
	XRT3D_TYPE_BAR,
	XRT3D_TYPE_SCATTER
};
static char * dXrt3dType[] = {
	"XRT3D_TYPE_SURFACE",
	"XRT3D_TYPE_BAR",
	"XRT3D_TYPE_SCATTER"
};

/* X Grid Lines */
static char * uXrt3dXGridLines[] = {
	"None",
	"XYPlane",
	"XZPlane",
	"XYPlane + XZPlane"
	};
static int xXrt3dXGridLines[] = {
	0,
	XRT3D_XY_PLANE,
	XRT3D_XZ_PLANE,
	XRT3D_XY_PLANE + XRT3D_XZ_PLANE
	};
static char * dXrt3dXGridLines[] = {
	"0",
	"XRT3D_XY_PLANE",
	"XRT3D_XZ_PLANE",
	"XRT3D_XY_PLANE + XRT3D_XZ_PLANE"
	};

/* Y Grid Lines */
static char * uXrt3dYGridLines[] = {
	"None",
	"XYPlane",
	"YZPlane",
	"XYPlane + YZPlane"
	};
static int xXrt3dYGridLines[] = {
	0,
	XRT3D_XY_PLANE,
	XRT3D_YZ_PLANE,
	XRT3D_XY_PLANE + XRT3D_YZ_PLANE
	};
static char * dXrt3dYGridLines[] = {
	"0",
	"XRT3D_XY_PLANE",
	"XRT3D_YZ_PLANE",
	"XRT3D_XY_PLANE + XRT3D_YZ_PLANE"
	};

/* Z Grid Lines */
static char * uXrt3dZGridLines[] = {
	"None",
	"XZPlane",
	"YZPlane",
	"XZPlane + YZPlane"
	};
static int xXrt3dZGridLines[] = {
	0,
	XRT3D_XZ_PLANE,
	XRT3D_YZ_PLANE,
	XRT3D_XZ_PLANE + XRT3D_YZ_PLANE
	};
static char * dXrt3dZGridLines[] = {
	"0",
	"XRT3D_XZ_PLANE",
	"XRT3D_YZ_PLANE",
	"XRT3D_XZ_PLANE + XRT3D_YZ_PLANE"
	};

/* Zone Method */
static char * uXrt3dZoneMethod[] = {
	"ZoneContours",
    "ZoneCells"
};
static int xXrt3dZoneMethod[] = {
	XRT3D_ZONE_CONTOURS,
    XRT3D_ZONE_CELLS
};
static char * dXrt3dZoneMethod[] = {
	"XRT3D_ZONE_CONTOURS",
    "XRT3D_ZONE_CELLS"
};

static char * uXrt3dDistnRange[] = {
    "DistnRangeData",
    "DistnRangeAll"
};
static int xXrt3dDistnRange[] = {
    XRT3D_DISTN_RANGE_DATA,
    XRT3D_DISTN_RANGE_ALL
};
static char * dXrt3dDistnRange[] = {
    "XRT3D_DISTN_RANGE_DATA",
    "XRT3D_DISTN_RANGE_ALL"
};

static char * uXrt3dImageLayout[] = {
    "ImagelayoutCentered",
    "ImagelayoutTiled"
};
static int xXrt3dImageLayout[] = {
    XRT3D_IMAGELAYOUT_CENTERED,
    XRT3D_IMAGELAYOUT_TILED
};
static char * dXrt3dImageLayout[] = {
    "XRT3D_IMAGELAYOUT_CENTERED",
    "XRT3D_IMAGELAYOUT_TILED"
};

#ifndef   RUNTIME

/*
 * ValuesOf functions for both enumerated types and other Xtypes
 */

int
UxValuesOfXrt3dAdjust(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dAdjust;
	*n = XtNumber(uXrt3dAdjust);
	return( *n);
}

int
UxValuesOfXrt3dAlign(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dAlign;
	*n = XtNumber(uXrt3dAlign);
	return( *n);
}

int
UxValuesOfXrt3dAnchor(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dAnchor;
	*n = XtNumber(uXrt3dAnchor);
	return( *n);
}

int
UxValuesOfXrt3dAnnoMethod(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dAnnoMethod;
	*n = XtNumber(uXrt3dAnnoMethod);
	return( *n);
}

int
UxValuesOfXrt3dAttachMethod(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dAttachMethod;
	*n = XtNumber(uXrt3dAttachMethod);
	return( *n);
}

int
UxValuesOfXrt3dBarFormat(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dBarFormat;
	*n = XtNumber(uXrt3dBarFormat);
	return( *n);
}

int
UxValuesOfXrt3dBorder(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dBorder;
	*n = XtNumber(uXrt3dBorder);
	return( *n);
}

int
UxValuesOfXrt3dContourStyles(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_3DCONTOURSTYLES);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrt3dData(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_3DDATA);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrt3dDistnMethod(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dDistnMethod;
	*n = XtNumber(uXrt3dDistnMethod);
	return( *n);
}

int
UxValuesOfXrt3dDistnTable(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0]=CGETS_KL(VALOF_3DDISTNTABLE);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrt3dDouble(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_3DDOUBLE);
	*s = values;
	*n = 1;

	return 0;
}

int
UxValuesOfXrt3dLegendStyle(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dLegendStyle;
	*n = XtNumber(uXrt3dLegendStyle);
	return( *n);
} 

int
UxValuesOfXrt3dPreviewMethod(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dPreviewMethod;
	*n = XtNumber(uXrt3dPreviewMethod);
	return( *n);
} 

int
UxValuesOfXrt3dPlane(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dPlane;
	*n = XtNumber(uXrt3dPlane);
	return( *n);
} 

int
UxValuesOfXrt3dProject(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dProject;
	*n = XtNumber(uXrt3dProject);
	return( *n);
} 

int
UxValuesOfXrt3dStrings(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_3DSTRING);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrt3dStrokeFont(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dStrokeFont;
	*n = XtNumber(uXrt3dStrokeFont);
	return( *n);
}

int
UxValuesOfXrt3dTextList(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_3DTEXTLIST);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrt3dType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dType;
	*n = XtNumber(uXrt3dType);
	return( *n);
}

int
UxValuesOfXrt3dValueLabels(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_3DVALUELABELS);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrt3dXYColors(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_3DXYCOLORS);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrt3dXGridLines(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dXGridLines;
	*n = XtNumber(uXrt3dXGridLines);
	return( *n);
}

int
UxValuesOfXrt3dYGridLines(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dYGridLines;
	*n = XtNumber(uXrt3dYGridLines);
	return( *n);
}

int
UxValuesOfXrt3dZGridLines(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dZGridLines;
	*n = XtNumber(uXrt3dZGridLines);
	return( *n);
}

int
UxValuesOfXrt3dZoneMethod(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrt3dZoneMethod;
	*n = XtNumber(uXrt3dZoneMethod);
	return( *n);
}

/*
 * Validate functions for both enumerated types and other Xtypes.
 * Use a general function for enumerated types.
 */

static int
ValidateXrt3dEnumType(rs, s, string_list, n)
swidget rs;
char	*s;
char	**string_list;
int		n;
{
	int i;

	if (s == NULL) {
		return ERROR;
	}
	for (i = 0; i < n; i++) {
		if (UxStrEqual(s, string_list[i])) {
			return NO_ERROR;
		}
	}
	return ERROR;
}

int
UxValidateXrt3dAdjust(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dAdjust,
								 XtNumber(uXrt3dAdjust)));
}

int
UxValidateXrt3dAlign(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dAlign,
								 XtNumber(uXrt3dAlign)));
}

int
UxValidateXrt3dAnchor(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dAnchor,
								 XtNumber(uXrt3dAnchor)));
}

int
UxValidateXrt3dAnnoMethod(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dAnnoMethod,
								 XtNumber(uXrt3dAnnoMethod)));
}

int
UxValidateXrt3dAttachMethod(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dAttachMethod,
								 XtNumber(uXrt3dAttachMethod)));
}

int
UxValidateXrt3dBarFormat(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dBarFormat,
								 XtNumber(uXrt3dBarFormat)));
}

int
UxValidateXrt3dBorder(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dBorder,
								 XtNumber(uXrt3dBorder)));
}

int
UxValidateXrt3dData(swgt, value)
swidget                         swgt;
char                            *value;
{
	Xrt3dData	*data;
	char		errbuf[100];
	int			status = ERROR;

	if (!UxStrEqual(value,"")) {
		data = Xrt3dMakeDataFromFile(value, errbuf);
		if (data) {
			status = NO_ERROR;	
			Xrt3dDestroyData(data, True);
		}
	}
	else {
		status = NO_ERROR;
	}

	return (status);
}

int
UxValidateXrt3dDistnMethod(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dDistnMethod,
								 XtNumber(uXrt3dDistnMethod)));
}

int
UxValidateXrt3dDistnTable(swgt, value)
swidget                         swgt;
char                            *value;
{
	Xrt3dDistnTable		*dtable;
	char				errbuf[100];
	int					status = ERROR;

	if (!UxStrEqual(value,"")) {
		dtable = Xrt3dDistnTableFromFile(value, errbuf);
		if (dtable) {
			status = NO_ERROR;
			Xrt3dFreeDistnTable(dtable);
		}
	}
	else {
		status = NO_ERROR;
	}

	return (status);
}

int
UxValidateXrt3dLegendStyle(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dLegendStyle,
								 XtNumber(uXrt3dLegendStyle)));
}

int
UxValidateXrt3dPreviewMethod(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dPreviewMethod,
								 XtNumber(uXrt3dPreviewMethod)));
}

int
UxValidateXrt3dPlane(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dPlane,
								 XtNumber(uXrt3dPlane)));
}

int
UxValidateXrt3dProject(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dProject,
								 XtNumber(uXrt3dProject)));
}

int
UxValidateXrt3dStrokeFont(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dStrokeFont,
								 XtNumber(uXrt3dStrokeFont)));
}

int
UxValidateXrt3dType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dType,
								 XtNumber(uXrt3dType)));
}

int
UxValidateXrt3dXGridLines(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dXGridLines,
								 XtNumber(uXrt3dXGridLines)));
}

int
UxValidateXrt3dYGridLines(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dYGridLines,
								 XtNumber(uXrt3dYGridLines)));
}

int
UxValidateXrt3dZGridLines(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dZGridLines,
								 XtNumber(uXrt3dZGridLines)));
}

int
UxValidateXrt3dZoneMethod(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dZoneMethod,
								 XtNumber(uXrt3dZoneMethod)));
}


int
UxValuesOfXrt3dDataStyles(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_3DDATASTYLES);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrt3dDistnRange(ulist, n)
char ***ulist;
int	*n;
{
	*ulist = uXrt3dDistnRange;
	*n = XtNumber(uXrt3dDistnRange);
	return(*n);
}


int
UxValuesOfXrt3dImage(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0]=CGETS_KL(VAOF_3DIMAGE);
	*s = values;
	*n = 1;
	return 0;
}


int
UxValuesOfXrt3dImageLayout(ulist, n)
char ***ulist;
int	*n;
{
	*ulist = uXrt3dImageLayout;
	*n = XtNumber(uXrt3dImageLayout);
	return(*n);
}

int
UxValuesOfXrt3dLineStyle(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0]=CGETS_KL(VAOF_3DLINESTYPE);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValidateXrt3dDistnRange(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dDistnRange,
								  XtNumber(uXrt3dDistnRange)));
}

int
UxValidateXrt3dImageLayout(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrt3dEnumType(rs, s, uXrt3dImageLayout,
								  XtNumber(uXrt3dImageLayout)));
}

#endif   /* ! RUNTIME */

/* Conversion functions TO and FROM UIMX */

static int
File_ContourStyles(swgt, file, cstyles, flag, xtype)
swidget				swgt;
char				**file;
Xrt3dContourStyle	***cstyles;
int					flag;
int					xtype;
{
	int				status;
	XrmValue        from, to;
	Boolean         stat;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		info = FindXrtStringInfo(File_ContourStyles, xtype, swgt);
		if (info) {
			*file = XtNewString(info->string);
		}
		else {
			*file = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {
		
		if (**file) {
		
			from.size = strlen( *file ) + 1;
			from.addr = *file;

			to.size = sizeof(Xrt3dContourStyle **);
			to.addr = (caddr_t) cstyles;

			stat = XtConvertAndStore (UxTopLevel,
									  XtRString, &from,
									  XtRXrt3dContourStyles, &to);
		
			if (stat) {
				RegisterXrtStringInfo(File_ContourStyles, xtype, swgt, *file);
				status = NO_ERROR;
			}
		}
		else {
			*cstyles = NULL;
			RegisterXrtStringInfo(File_ContourStyles, xtype, swgt, "");
			status = NO_ERROR;
		}
	}

	return(status);
}

static int
File_DistnTable(swgt, file, dtable, flag, xtype)
swidget				swgt;
char				**file;
Xrt3dDistnTable		**dtable;
int					flag;
int					xtype;
{
	int				status;
	XrmValue        from, to;
	Boolean         stat;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		info = FindXrtStringInfo(File_DistnTable, xtype, swgt);
		if (info) {
			*file = XtNewString(info->string);
		}
		else {
			*file = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {
		
		if (**file) {
		
			from.size = strlen( *file ) + 1;
			from.addr = *file;

			to.size = sizeof(Xrt3dDistnTable *);
			to.addr = (caddr_t) dtable;

			stat = XtConvertAndStore (UxTopLevel,
									  XtRString, &from,
									  XtRXrt3dDistnTable, &to);
		
			if (stat) {
				RegisterXrtStringInfo(File_DistnTable, xtype, swgt, *file);
				status = NO_ERROR;
			}
		}
		else {
			*dtable = NULL;
			RegisterXrtStringInfo(File_DistnTable, xtype, swgt, "");
			status = NO_ERROR;
		}
	}
	
	return(status);
}

static int
String_ValueLabels(swgt, string, vlabels, flag, xtype)
swidget				swgt;
char				**string;
Xrt3dValueLabel		***vlabels;
int					flag;
int					xtype;
{
	int			i;
	int			status;
	XrmValue	from, to;
	Boolean		stat;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		from.size = sizeof(Xrt3dValueLabel **);
		from.addr = (caddr_t) vlabels;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XtRXrt3dValueLabels, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn "((2.0 "value") (3.0 "three")(-4.2 "a string") ...)"
		 *  into a list of value label structures.
		 */

		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(Xrt3dValueLabel **);
		to.addr = (caddr_t) vlabels;

		stat = XtConvertAndStore (UxTopLevel,
								  XtRString, &from,
								  XtRXrt3dValueLabels, &to);


		if (stat) {
			if (*vlabels) {
				for (i = 0; (*vlabels)[i]; i++) {
				}
			}
			status = NO_ERROR;
		}
	}


	return(status);
}

static int
String_XYColors(swgt, string, xycolor, flag, xtype)
swidget			swgt;
char			**string;
Xrt3dXYColor	***xycolor;
int				flag;
int				xtype;
{
	int			i;
	int			status;
	XrmValue	from, to;
	Boolean		stat;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		from.size = sizeof(Xrt3dXYColor **);
		from.addr = (caddr_t) xycolor;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XtRXrt3dXYColors, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn "((1 1 "red") (3 -1 "green")(-1 5 "#FF23AC") ...)"
		 *  into a list of xycolors structures.
		 */
		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(Xrt3dXYColor **);
		to.addr = (caddr_t) xycolor;

		stat = XtConvertAndStore (UxTopLevel,
								  XtRString, &from,
								  XtRXrt3dXYColors, &to);


		if (stat) {
			if (*xycolor) {
				for (i = 0; (*xycolor)[i]; i++) {
				}
			}

			status = NO_ERROR;
		}
	}

	return(status);
}

static int
File_SurfaceData(swgt, file, data, flag, xtype)
swidget		swgt;
char		**file;
Xrt3dData	**data;
int			flag;
int			xtype;
{
	int				status;
	XrmValue        from, to;
	Boolean         stat;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {

		info = FindXrtStringInfo(File_SurfaceData, xtype, swgt);
		if (info) {
			*file = XtNewString(info->string);
		}
		else {
			*file = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {

		if (**file) {
		
			from.size = strlen( *file ) + 1;
			from.addr = *file;

			to.size = sizeof(Xrt3dData *);
			to.addr = (caddr_t) data;

			stat = XtConvertAndStore (UxTopLevel,
									  XtRString, &from,
									  XtRXrt3dData, &to);
		
			if (stat) {
				RegisterXrtStringInfo(File_SurfaceData, xtype, swgt, *file);
				status = NO_ERROR;
			}
		}
		else {
			*data = NULL;
			RegisterXrtStringInfo(File_SurfaceData, xtype, swgt, "");
			status = NO_ERROR;
		}
	}

	return(status);
}

static int
String_TextList(swgt, string, textlist, flag, xtype)
swidget			swgt;
char			**string;
WidgetList		*textlist;
int				flag;
int				xtype;
{
	int				i;
	int				status;
	XrmValue		from, to;
	Boolean			stat;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		from.size = sizeof(WidgetList);
		from.addr = (caddr_t) textlist;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XtRXrt3dTextList, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(WidgetList);
		to.addr = (caddr_t) textlist;

		status = XtConvertAndStore (UxTopLevel,
									XtRString, &from,
									XtRXrt3dTextList, &to);
	}

	return(status);
}

static int
File_ZoneData(swgt, file, data, flag, xtype)
swidget		swgt;
char		**file;
Xrt3dData	**data;
int			flag;
int			xtype;
{
	int				status;
	XrmValue        from, to;
	Boolean         stat;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		info = FindXrtStringInfo(File_ZoneData, xtype, swgt);
		if (info) {
			*file = XtNewString(info->string);
		}
		else {
			*file = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {
		
		if (**file) {
		
			from.size = strlen( *file ) + 1;
			from.addr = *file;

			to.size = sizeof(Xrt3dData *);
			to.addr = (caddr_t) data;

			stat = XtConvertAndStore (UxTopLevel,
									  XtRString, &from,
									  XtRXrt3dData, &to);
		
			if (stat) {
				RegisterXrtStringInfo(File_ZoneData, xtype, swgt, *file);
				status = NO_ERROR;
			}
		}
		else {
			*data = NULL;
			RegisterXrtStringInfo(File_ZoneData, xtype, swgt, "");
			status = NO_ERROR;
		}
	}

	return(status);
}

static int
Xrt3d_String_Strings(swgt, string, strings, flag, xtype)
swidget		swgt;
char		**string;
char		***strings;
int			flag;
int			xtype;
{
	int				i;
	int				status;
	XrmValue        from, to;
	Boolean         stat;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		from.size = sizeof(char **);
		from.addr = (caddr_t) strings;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XtRXrt3dStrings, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn "( \"this\" \"is\" \"a\" \"list\" \"of\" \"strings\" ...)"
		 *  into something interesting.
		 */

		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(char **);
		to.addr = (caddr_t) strings;

		stat = XtConvertAndStore (UxTopLevel,
								  XtRString, &from,
								  XtRXrt3dStrings, &to);
		
		if (stat) {
			if (*strings) {
				for (i = 0; (*strings)[i]; i++) {
				}
			}

			status = NO_ERROR;
		}
	}

	return(status);
}

static int
String_DataStyles(swgt, string, dstyles, flag, xtype)
swidget			swgt;
char			**string;
Xrt3dDataStyle	***dstyles;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {

		from.size = sizeof(Xrt3dDataStyle **);
		from.addr = (caddr_t) dstyles;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XtRXrt3dDataStyles, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn a data styles string into a data styles structure
		 */

		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(Xrt3dDataStyle **);
		to.addr = (caddr_t) dstyles;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XtRXrt3dDataStyles, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

static int
String_Image(swgt, string, image, flag, xtype)
swidget			swgt;
char			**string;
Xrt3dImage		*image;
int				flag;
int				xtype;
{
	int			i;
	int			status;
	XrmValue	from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  Turn a Image into a string.
		 */

		from.size = sizeof(Xrt3dImage);
		from.addr = (caddr_t) image;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XtRXrt3dImage, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn string into a Image.
		 */
		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(Xrt3dImage);
		to.addr = (caddr_t) image;

		if (XtConvertAndStore (UxTopLevel,
							   XtRString, &from,
							   XtRXrt3dImage, &to))
		{
			status = NO_ERROR;
		}
	}

	return(status);
}

static int
String_LineStyle(swgt, string, lstyle, flag, xtype)
swidget			swgt;
char			**string;
Xrt3dLineStyle	**lstyle;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {

		from.size = sizeof(Xrt3dLineStyle *);
		from.addr = (caddr_t) lstyle;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XtRXrt3dLineStyle, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn a data styles string into a data styles structure
		 */

		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(Xrt3dLineStyle *);
		to.addr = (caddr_t) lstyle;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XtRXrt3dLineStyle, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

void
Xrt3d_enum()
{
	UxXT_Xrt3dAdjust = UxAddEnumType("Xrt3dAdjust", 
									 sizeof(int),
									 (unsigned char *)xXrt3dAdjust, 
									 uXrt3dAdjust, 
									 dXrt3dAdjust, 
									 XtNumber(uXrt3dAdjust), 
									 UxStringToIntEnum); 
	UxXT_Xrt3dAlign = UxAddEnumType("Xrt3dAlign", 
									sizeof(int),
									(unsigned char *)xXrt3dAlign, 
									uXrt3dAlign, 
									dXrt3dAlign, 
									XtNumber(uXrt3dAlign), 
									UxStringToIntEnum); 
	UxXT_Xrt3dAnchor = UxAddEnumType("Xrt3dAnchor", 
									 sizeof(int),
									 (unsigned char *)xXrt3dAnchor, 
									 uXrt3dAnchor, 
									 dXrt3dAnchor, 
									 XtNumber(uXrt3dAnchor), 
									 UxStringToIntEnum); 
	UxXT_Xrt3dAnnoMethod = UxAddEnumType("Xrt3dAnnoMethod", 
										 sizeof(int),
										 (unsigned char *)xXrt3dAnnoMethod, 
										 uXrt3dAnnoMethod, 
										 dXrt3dAnnoMethod, 
										 XtNumber(uXrt3dAnnoMethod), 
										 UxStringToIntEnum); 
	UxXT_Xrt3dAttachMethod = UxAddEnumType("Xrt3dAttachMethod", 
										   sizeof(int),
										   (unsigned char *)xXrt3dAttachMethod, 
										   uXrt3dAttachMethod, 
										   dXrt3dAttachMethod, 
										   XtNumber(uXrt3dAttachMethod), 
										   UxStringToIntEnum); 
	UxXT_Xrt3dBarFormat = UxAddEnumType("Xrt3dBarFormat", 
										sizeof(int),
										(unsigned char *)xXrt3dBarFormat, 
										uXrt3dBarFormat, 
										dXrt3dBarFormat, 
										XtNumber(uXrt3dBarFormat), 
										UxStringToIntEnum); 
	UxXT_Xrt3dBorder = UxAddEnumType("Xrt3dBorder", 
									 sizeof(int),
									 (unsigned char *)xXrt3dBorder, 
									 uXrt3dBorder, 
									 dXrt3dBorder, 
									 XtNumber(uXrt3dBorder), 
									 UxStringToIntEnum); 
	UxXT_Xrt3dDistnMethod = UxAddEnumType("Xrt3dDistnMethod", 
										  sizeof(int),
										  (unsigned char *)xXrt3dDistnMethod, 
										  uXrt3dDistnMethod, 
										  dXrt3dDistnMethod, 
										  XtNumber(uXrt3dDistnMethod), 
										  UxStringToIntEnum); 
	UxXT_Xrt3dLegendStyle = UxAddEnumType("Xrt3dLegendStyle", 
										  sizeof(int),
										  (unsigned char *)xXrt3dLegendStyle, 
										  uXrt3dLegendStyle, 
										  dXrt3dLegendStyle, 
										  XtNumber(uXrt3dLegendStyle), 
										  UxStringToIntEnum); 
	UxXT_Xrt3dPreviewMethod = UxAddEnumType("Xrt3dPreviewMethod", 
										  sizeof(int),
										  (unsigned char *)xXrt3dPreviewMethod, 
										  uXrt3dPreviewMethod, 
										  dXrt3dPreviewMethod, 
										  XtNumber(uXrt3dPreviewMethod), 
										  UxStringToIntEnum); 
	UxXT_Xrt3dPlane = UxAddEnumType("Xrt3dPlane", 
									sizeof(int),
									(unsigned char *)xXrt3dPlane, 
									uXrt3dPlane, 
									dXrt3dPlane, 
									XtNumber(uXrt3dPlane), 
									UxStringToIntEnum); 
	UxXT_Xrt3dProject = UxAddEnumType("Xrt3dProject", 
									  sizeof(int),
									  (unsigned char *)xXrt3dProject, 
									  uXrt3dProject, 
									  dXrt3dProject, 
									  XtNumber(uXrt3dProject), 
									  UxStringToIntEnum); 
	UxXT_Xrt3dStrokeFont = UxAddEnumType("Xrt3dStrokeFont", 
										 sizeof(int),
										 (unsigned char *)xXrt3dStrokeFont, 
										 uXrt3dStrokeFont, 
										 dXrt3dStrokeFont, 
										 XtNumber(uXrt3dStrokeFont), 
										 UxStringToIntEnum); 
	UxXT_Xrt3dType = UxAddEnumType("Xrt3dType", 
								   sizeof(int),
								   (unsigned char *)xXrt3dType, 
								   uXrt3dType, 
								   dXrt3dType, 
								   XtNumber(uXrt3dType), 
								   UxStringToIntEnum); 
	UxXT_Xrt3dXGridLines = UxAddEnumType("Xrt3dXGridLines", 
										 sizeof(int),
										 (unsigned char *)xXrt3dXGridLines, 
										 uXrt3dXGridLines, 
										 dXrt3dXGridLines, 
										 XtNumber(uXrt3dXGridLines), 
										 UxStringToIntEnum); 
	UxXT_Xrt3dYGridLines = UxAddEnumType("Xrt3dYGridLines", 
										 sizeof(int),
										 (unsigned char *)xXrt3dYGridLines, 
										 uXrt3dYGridLines, 
										 dXrt3dYGridLines, 
										 XtNumber(uXrt3dYGridLines), 
										 UxStringToIntEnum); 
	UxXT_Xrt3dZGridLines = UxAddEnumType("Xrt3dZGridLines", 
										 sizeof(int),
										 (unsigned char *)xXrt3dZGridLines, 
										 uXrt3dZGridLines, 
										 dXrt3dZGridLines, 
										 XtNumber(uXrt3dZGridLines), 
										 UxStringToIntEnum); 
	UxXT_Xrt3dZoneMethod = UxAddEnumType("Xrt3dZoneMethod", 
										 sizeof(int),
										 (unsigned char *)xXrt3dZoneMethod, 
										 uXrt3dZoneMethod, 
										 dXrt3dZoneMethod, 
										 XtNumber(uXrt3dZoneMethod), 
										 UxStringToIntEnum); 

	UxXT_Xrt3dDistnRange = UxAddEnumType("Xrt3dDistnRange", 
										 sizeof(int),
										 (unsigned char *)xXrt3dDistnRange,
										 uXrt3dDistnRange,
										 dXrt3dDistnRange,
										 XtNumber(uXrt3dDistnRange),
										 UxStringToIntEnum);
	UxXT_Xrt3dImageLayout = UxAddEnumType("Xrt3dImageLayout", 
										  sizeof(int),
										  (unsigned char *)xXrt3dImageLayout,
										  uXrt3dImageLayout,
										  dXrt3dImageLayout,
										  XtNumber(uXrt3dImageLayout),
										  UxStringToIntEnum);
}

/* Add new Xrt/3d xtypes */

void
Xrt3d_xtypes()
{
	/*
	 * Initialize widget class inorder to initialize converters
	 */

	XtInitializeWidgetClass(xtXrt3dWidgetClass);
	XrtInitCommonXtypes();
	Xrt3dSetReverseConverters();

	UxXT_Xrt3dContourStyles = UxAddXtype(XtRXrt3dContourStyles,
										 sizeof(Xrt3dContourStyle **));
	UxXT_Xrt3dDistnTable    = UxAddXtype(XtRXrt3dDistnTable,
										 sizeof(Xrt3dDistnTable *));
	UxXT_Xrt3dDouble		= UxAddXtype(XtRXrt3dDouble, sizeof(double));
	UxXT_Xrt3dSurfaceData   = UxAddXtype(XtRXrt3dData, sizeof(Xrt3dData *));
	UxXT_Xrt3dStrings       = UxAddXtype(XtRXrt3dStrings, sizeof(char **));
	UxXT_Xrt3dTextList      = UxAddXtype(XtRXrt3dTextList, sizeof(WidgetList));
	UxXT_Xrt3dValueLabels   = UxAddXtype(XtRXrt3dValueLabels,
										 sizeof(Xrt3dValueLabel **));
	UxXT_Xrt3dXYColors      = UxAddXtype(XtRXrt3dXYColors,
										 sizeof(Xrt3dXYColor **));
	UxXT_Xrt3dZoneData      = UxAddXtype(XtRXrt3dData, sizeof(Xrt3dData *));

	UxXT_Xrt3dDataStyles	= UxAddXtype(XtRXrt3dDataStyles,
										 sizeof(Xrt3dDataStyle **));
	UxXT_Xrt3dImage	= UxAddXtype(XtRXrt3dImage, sizeof(Xrt3dImage));
	UxXT_Xrt3dLineStyle	= UxAddXtype(XtRXrt3dLineStyle,
									 sizeof(Xrt3dLineStyle *));
	/* Add new converters for Xtypes */

	UxAddConv(UxUT_string, UxXT_Xrt3dContourStyles, File_ContourStyles);
	UxAddConv(UxUT_string, UxXT_Xrt3dDistnTable,	File_DistnTable);
	UxAddConv(UxUT_string, UxXT_Xrt3dStrings,		Xrt3d_String_Strings);
	UxAddConv(UxUT_string, UxXT_Xrt3dSurfaceData,	File_SurfaceData);
	UxAddConv(UxUT_string, UxXT_Xrt3dTextList,		String_TextList);
	UxAddConv(UxUT_string, UxXT_Xrt3dValueLabels,	String_ValueLabels);
	UxAddConv(UxUT_string, UxXT_Xrt3dXYColors,		String_XYColors);
	UxAddConv(UxUT_string, UxXT_Xrt3dZoneData,		File_ZoneData);
  	UxAddConv(UxUT_string, UxXT_Xrt3dDataStyles, 	String_DataStyles);
  	UxAddConv(UxUT_string, UxXT_Xrt3dImage, 		String_Image);
  	UxAddConv(UxUT_string, UxXT_Xrt3dLineStyle, 	String_LineStyle);

}

/*
 * Special get function to replace UxGetProp in UxXrt3d.h
 * for resources of type "double". UxGetProp and UxDDGetProp
 * always return XtArgVal but XRT/3d is somewhat asymetric
 * when it comes to putting and getting doubles. This funtion
 * gets the double (XRT style) then returns a pointer to it
 * which the UxGet... macro dereferences using Xrt3dArgValToDouble -jw
 */

double
UxXrt3dArgValToDouble(v)
XtArgVal v;
{
	/*  XRT3dArgValToDouble() does not work LP64 platforms.  */
	/*  This is a workaround for it. */
	return(*(double *)v);
}

XtArgVal
UxGetXrt3dDoubleProp(sw, res)
swidget	sw;
char	*res;
{
	Widget      	w;
	static double  	value = (double) 0.0;

	if (UxIsSwidget(sw) && (w = (Widget) UxGetWidget(sw)) != NULL) {
		XtVaGetValues (w, res, &value, NULL);
	}

	return (XtArgVal)&value;
}

