/* $AstecId: XrtFld_restb.c,v 1.10 1999/10/13 11:41:27 nakayama Exp nakayama $ */
/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/field widget UIM/X integration file
 *
 *  SCCSID -- @(#)XrtFld_restb.c	1.34    99/02/01 KL Group Inc.
 */

/*****************************************************************************
				XrtFld_restb.c
*****************************************************************************/

#include <Xm/XrtField.h>		/* Widget-class public header file */
#include <UxLib.h>

#ifdef DESIGN_TIME

#include "veos.h"       		/* Visual Edge object system */
#include "valuesOf.h"   		/* ValuesOf & Validator functions */

#include "XrtFld.cl.h"

#else

#include "stub_runtime.h"

#endif /* DESIGN_TIME */

#include "XrtFld_xtype.h"
#include "Xrt_restable.h"

extern swidget UxGUIPopupColorView();

static int   XrtFldPutDouble();
static float XrtFldGetDouble();

/* Resource table for XRT field widget */

XrtFldResourceDesc xrtfld_res_table[] = {
{
	"xrtFldAllowNull",
	"RD_xrtFldAllowNull",
	XmNxrtFldAllowNull,
	"XmNxrtFldAllowNull",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldAllowNull),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumAllowNull,
	FALSE,
	TRUE
},
{
	"xrtFldCase",
	"RD_xrtFldCase",
	XmNxrtFldCase,
	"XmNxrtFldCase",
	XRTFLD_CASE,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldCase),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumCase,
	FALSE,
	TRUE
},
{
	"xrtFldCompleteCallback",
	"RD_xrtFldCompleteCallback",
	XmNxrtFldCompleteCallback,
	"XmNxrtFldCompleteCallback",
	XRTFLD_CALLBACK,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldCompleteCallback),
	XRTDIV_BEHAVIOR,
	XRTPASS0,
	NULL,
	xrtFldResNumCompleteCallback,
	FALSE,
	TRUE
},
{
	"xrtFldConvertCallback",
	"RD_xrtFldConvertCallback",
	XmNxrtFldConvertCallback,
	"XmNxrtFldConvertCallback",
	XRTFLD_CALLBACK,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldConvertCallback),
	XRTDIV_BEHAVIOR,
	XRTPASS0,
	NULL,
	xrtFldResNumConvertCallback,
	FALSE,
	TRUE
},
{
	"xrtFldDefaultBackground",
	"RD_xrtFldDefaultBackground",
	XmNxrtFldDefaultBackground,
	"XmNxrtFldDefaultBackground",
	XRTFLD_COLOR,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldDefaultBackground),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumDefaultBackground,
	FALSE,
	TRUE
},
{
	"xrtFldDefaultForeground",
	"RD_xrtFldDefaultForeground",
	XmNxrtFldDefaultForeground,
	"XmNxrtFldDefaultForeground",
	XRTFLD_COLOR,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldDefaultForeground),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumDefaultForeground,
	FALSE,
	TRUE
},
{
	"xrtFldDisplayFormat",
	"RD_xrtFldDisplayFormat",
	XmNxrtFldDisplayFormat,
	"XmNxrtFldDisplayFormat",
	XRTFLD_FORMAT,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldDisplayFormat),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumDisplayFormat,
	FALSE,
	TRUE
},
{
	"xrtFldDisplayHorizScrollBar",
	"RD_xrtFldDisplayHorizScrollBar",
	XmNxrtFldDisplayHorizScrollBar,
	"XmNxrtFldDisplayHorizScrollBar",
	XRTFLD_DISP_SCROLLBAR,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldDisplayHorizScrollBar),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumDisplayHorizScrollBar,
	FALSE,
	TRUE
},
{
	"xrtFldDisplayVertScrollBar",
	"RD_xrtFldDisplayVertScrollBar",
	XmNxrtFldDisplayVertScrollBar,
	"XmNxrtFldDisplayVertScrollBar",
	XRTFLD_DISP_SCROLLBAR,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldDisplayVertScrollBar),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumDisplayVertScrollBar,
	FALSE,
	TRUE
},
{
	"xrtFldEditFormat",
	"RD_xrtFldEditFormat",
	XmNxrtFldEditFormat,
	"XmNxrtFldEditFormat",
	XRTFLD_FORMAT,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldEditFormat),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumEditFormat,
	FALSE,
	TRUE
},
{
	"xrtFldEnterCallback",
	"RD_xrtFldEnterCallback",
	XmNxrtFldEnterCallback,
	"XmNxrtFldEnterCallback",
	XRTFLD_CALLBACK,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldEnterCallback),
	XRTDIV_BEHAVIOR,
	XRTPASS0,
	NULL,
	xrtFldResNumEnterCallback,
	FALSE,
	TRUE
},
{
	"xrtFldErrorBackground",
	"RD_xrtFldErrorBackground",
	XmNxrtFldErrorBackground,
	"XmNxrtFldErrorBackground",
	XRTFLD_COLOR,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldErrorBackground),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumErrorBackground,
	FALSE,
	TRUE
},
{
	"xrtFldErrorCallback",
	"RD_xrtFldErrorCallback",
	XmNxrtFldErrorCallback,
	"XmNxrtFldErrorCallback",
	XRTFLD_CALLBACK,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldErrorCallback),
	XRTDIV_BEHAVIOR,
	XRTPASS0,
	NULL,
	xrtFldResNumErrorCallback,
	FALSE,
	TRUE
},
{
	"xrtFldErrorForeground",
	"RD_xrtFldErrorForeground",
	XmNxrtFldErrorForeground,
	"XmNxrtFldErrorForeground",
	XRTFLD_COLOR,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldErrorForeground),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumErrorForeground,
	FALSE,
	TRUE
},
{
	"xrtFldErrorString",
	"RD_xrtFldErrorString",
	XmNxrtFldErrorString,
	"XmNxrtFldErrorString",
	XRTFLD_XMSTRING,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldErrorString),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumErrorString,
	FALSE,
	TRUE
},
{
	"xrtFldExitCallback",
	"RD_xrtFldExitCallback",
	XmNxrtFldExitCallback,
	"XmNxrtFldExitCallback",
	XRTFLD_CALLBACK,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldExitCallback),
	XRTDIV_BEHAVIOR,
	XRTPASS0,
	NULL,
	xrtFldResNumExitCallback,
	FALSE,
	TRUE
},
{
	"xrtFldInvalidChars",
	"RD_xrtFldInvalidChars",
	XmNxrtFldInvalidChars,
	"XmNxrtFldInvalidChars",
	XRTFLD_STRING,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldInvalidChars),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumInvalidChars,
	FALSE,
	TRUE
},
{
	"xrtFldIsDefault",
	"RD_xrtFldIsDefault",
	XmNxrtFldIsDefault,
	"XmNxrtFldIsDefault",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldIsDefault),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumIsDefault,
	TRUE,
	TRUE
},
{
	"xrtFldIsModified",
	"RD_xrtFldIsModified",
	XmNxrtFldIsModified,
	"XmNxrtFldIsModified",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldIsModified),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumIsModified,
	TRUE,
	TRUE
},
{
	"xrtFldIsValid",
	"RD_xrtFldIsValid",
	XmNxrtFldIsValid,
	"XmNxrtFldIsValid",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldIsValid),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumIsValid,
	TRUE,
	TRUE
},
{
	"xrtFldLastError",
	"RD_xrtFldLastError",
	XmNxrtFldLastError,
	"XmNxrtFldLastError",
	XRTFLD_ERROR,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldLastError),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumLastError,
	TRUE,
	TRUE
},
{
	"xrtFldMask",
	"RD_xrtFldMask",
	XmNxrtFldMask,
	"XmNxrtFldMask",
	XRTFLD_STRING,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldMask),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumMask,
	FALSE,
	TRUE
},
{
	"xrtFldMaskChars",
	"RD_xrtFldMaskChars",
	XmNxrtFldMaskChars,
	"XmNxrtFldMaskChars",
	XRTFLD_STRING,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldMaskChars),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumMaskChars,
	FALSE,
	TRUE
},
{
	"xrtFldMatchMask",
	"RD_xrtFldMatchMask",
	XmNxrtFldMatchMask,
	"XmNxrtFldMatchMask",
	XRTFLD_MATCH_MASK,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldMatchMask),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumMatchMask,
	FALSE,
	TRUE
},
{
	"xrtFldMatchPickList",
	"RD_xrtFldMatchPickList",
	XmNxrtFldMatchPickList,
	"XmNxrtFldMatchPickList",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldMatchPickList),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumMatchPickList,
	FALSE,
	TRUE
},
{
	"xrtFldMaxLineLength",
	"RD_xrtFldMaxLineLength",
	XmNxrtFldMaxLineLength,
	"XmNxrtFldMaxLineLength",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldMaxLineLength),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumMaxLineLength,
	FALSE,
	TRUE
},
{
	"xrtFldMaxLines",
	"RD_xrtFldMaxLines",
	XmNxrtFldMaxLines,
	"XmNxrtFldMaxLines",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldMaxLines),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumMaxLines,
	FALSE,
	TRUE
},
{ /* pds*/
	"xrtFldMenuAlignment",
	"RD_xrtFldMenuAlignment",
	XmNxrtFldMenuAlignment,
	"XmNxrtFldMenuAlignment",
	XRTFLD_MENU_ALIGNMENT,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldMenuAlignment),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumMenuAlignment,
	FALSE,
	TRUE
},
{
	"xrtFldMenuCallback",
	"RD_xrtFldMenuCallback",
	XmNxrtFldMenuCallback,
	"XmNxrtFldMenuCallback",
	XRTFLD_CALLBACK,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldMenuCallback),
	XRTDIV_BEHAVIOR,
	XRTPASS0,
	NULL,
	xrtFldResNumMenuCallback,
	FALSE,
	TRUE
},
{
	"xrtFldMenuList",
	"RD_xrtFldMenuList",
	XmNxrtFldMenuList,
	"XmNxrtFldMenuList",
	XRTFLD_WIDGET,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldMenuList),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumMenuList,
	TRUE,
	TRUE
},
{
	"xrtFldOverstrike",
	"RD_xrtFldOverstrike",
	XmNxrtFldOverstrike,
	"XmNxrtFldOverstrike",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldOverstrike),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumOverstrike,
	FALSE,
	TRUE
},
{
	"xrtFldPickListIndex",
	"RD_xrtFldPickListIndex",
	XmNxrtFldPickListIndex,
	"XmNxrtFldPickListIndex",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldPickListIndex),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumPickListIndex,
	FALSE,
	TRUE
},
{
	"xrtFldPromptString",
	"RD_xrtFldPromptString",
	XmNxrtFldPromptString,
	"XmNxrtFldPromptString",
	XRTFLD_XMSTRING,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldPromptString),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumPromptString,
	FALSE,
	TRUE
},
{
	"xrtFldRange",
	"RD_xrtFldRange",
	XmNxrtFldRange,
	"XmNxrtFldRange",
	XRTFLD_RANGE,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldRange),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumRange,
	FALSE,
	TRUE
},
{ /* pds */
	"xrtFldSearchPolicy",
	"RD_xrtFldSearchPolicy",
	XmNxrtFldSearchPolicy,
	"XmNxrtFldSearchPolicy",
	XRTFLD_SEARCH_POLICY,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldSearchPolicy),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSearchPolicy,
	FALSE,
	TRUE
},
{
	"xrtFldSpinCallback",
	"RD_xrtFldSpinCallback",
	XmNxrtFldSpinCallback,
	"XmNxrtFldSpinCallback",
	XRTFLD_CALLBACK,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldSpinCallback),
	XRTDIV_BEHAVIOR,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinCallback,
	FALSE,
	TRUE
},
{
	"xrtFldValidChars",
	"RD_xrtFldValidChars",
	XmNxrtFldValidChars,
	"XmNxrtFldValidChars",
	XRTFLD_STRING,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldValidChars),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumValidChars,
	FALSE,
	TRUE
},
{
	"xrtFldValidateCallback",
	"RD_xrtFldValidateCallback",
	XmNxrtFldValidateCallback,
	"XmNxrtFldValidateCallback",
	XRTFLD_CALLBACK,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldValidateCallback),
	XRTDIV_BEHAVIOR,
	XRTPASS0,
	NULL,
	xrtFldResNumValidateCallback,
	FALSE,
	TRUE
},
{
	"xrtFldVisibleItemCount",
	"RD_xrtFldVisibleItemCount",
	XmNxrtFldVisibleItemCount,
	"XmNxrtFldVisibleItemCount",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldVisibleItemCount),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumVisibleItemCount,
	FALSE,
	TRUE
},
{
	"xrtFldAutoComplete",
	"RD_xrtFldAutoComplete",
	XmNxrtFldAutoComplete,
	"XmNxrtFldAutoComplete",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldAutoComplete),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumAutoComplete,
	FALSE,
	TRUE
},
{
	"xrtFldCreatePopups",
	"RD_xrtFldCreatePopups",
	XmNxrtFldCreatePopups,
	"XmNxrtFldCreatePopups",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldCreatePopups),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumCreatePopups,
	FALSE,
	TRUE
},
{
	"xrtFldPopupType",
	"RD_xrtFldPopupType",
	XmNxrtFldPopupType,
	"XmNxrtFldPopupType",
	XRTFLD_POPUP_TYPE,
	{0, 0, 0},
	Offset(UxXrtFieldClass, XrtField.RD_xrtFldPopupType),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumPopupType,
	FALSE,
	TRUE
}
};

int numfld_res = (sizeof(xrtfld_res_table) / sizeof(xrtfld_res_table[0]));

/* Resource table for XRT string field widget */

XrtFldResourceDesc xrtfldstr_res_table[] = {
{
	"xrtStringFldDefaultValue",
	"RD_xrtFldDefaultValue",
	XmNxrtFldDefaultValue,
	"XmNxrtFldDefaultValue",
	XRTFLD_STRING,
	{0, 0, 0},
	Offset(UxXrtStringFieldClass, XrtStringField.RD_xrtFldDefaultValue),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtStringFldResNumDefaultValue,
	FALSE,
	TRUE
},
{
	"xrtStringFldInternalValue",
	"RD_xrtFldInternalValue",
	XmNxrtFldInternalValue,
	"XmNxrtFldInternalValue",
	XRTFLD_STRING,
	{0, 0, 0},
	Offset(UxXrtStringFieldClass, XrtStringField.RD_xrtFldInternalValue),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtStringFldResNumInternalValue,
	FALSE,
	TRUE
},
{
	"xrtStringFldMax",
	"RD_xrtFldMax",
	XmNxrtFldMax,
	"XmNxrtFldMax",
	XRTFLD_STRING,
	{0, 0, 0},
	Offset(UxXrtStringFieldClass, XrtStringField.RD_xrtFldMax),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtStringFldResNumMax,
	FALSE,
	TRUE
},
{
	"xrtStringFldMin",
	"RD_xrtFldMin",
	XmNxrtFldMin,
	"XmNxrtFldMin",
	XRTFLD_STRING,
	{0, 0, 0},
	Offset(UxXrtStringFieldClass, XrtStringField.RD_xrtFldMin),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtStringFldResNumMin,
	FALSE,
	TRUE
},
{
	"xrtStringFldPickList",
	"RD_xrtFldPickList",
	XmNxrtFldPickList,
	"XmNxrtFldPickList",
	XRTFLD_MULTI_TYPE_LIST,
	{0, 0, 0},
	Offset(UxXrtStringFieldClass, XrtStringField.RD_xrtFldPickList),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtStringFldResNumPickList,
	FALSE,
	TRUE
}
};

int numfldstr_res =
(sizeof(xrtfldstr_res_table) / sizeof(xrtfldstr_res_table[0]));

/* Resource table for XRT date field widget */

XrtFldResourceDesc xrtflddat_res_table[] = {
{
	"xrtDateFldCounterIncrement",
	"RD_xrtFldCounterIncrement",
	XmNxrtFldCounterIncrement,
	"XmNxrtFldCounterIncrement",
	XRTFLD_DATE_INCREMENT,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldCounterIncrement),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtDateFldResNumCounterIncrement,
	FALSE,
	TRUE
},
{
	"xrtDateFldDefaultValue",
	"RD_xrtFldDefaultValue",
	XmNxrtFldDefaultValue,
	"XmNxrtFldDefaultValue",
	XRTFLD_DATE,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldDefaultValue),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtDateFldResNumDefaultValue,
	FALSE,
	TRUE
},
{
	"xrtDateFldDisplayFormatString",
	"RD_xrtFldDisplayFormatString",
	XmNxrtFldDisplayFormatString,
	"XmNxrtFldDisplayFormatString",
	XRTFLD_STRING,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldDisplayFormatString),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtDateFldResNumDisplayFormatString,
	FALSE,
	TRUE
},
{
	"xrtDateFldEnterFormatList",
	"RD_xrtFldEnterFormatList",
	XmNxrtFldEnterFormatList,
	"XmNxrtFldEnterFormatList",
	XRTFLD_STRING_TABLE,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldEnterFormatList),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtDateFldResNumEnterFormatList,
	FALSE,
	TRUE
},
{
	"xrtDateFldInternalValue",
	"RD_xrtFldInternalValue",
	XmNxrtFldInternalValue,
	"XmNxrtFldInternalValue",
	XRTFLD_DATE,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldInternalValue),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtDateFldResNumInternalValue,
	FALSE,
	TRUE
},
{
	"xrtDateFldInternalValueLong",
	"RD_xrtFldInternalValueLong",
	XmNxrtFldInternalValueLong,
	"XmNxrtFldInternalValueLong",
	XRTFLD_LONG,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldInternalValueLong),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtDateFldResNumInternalValueLong,
	FALSE,
	TRUE
},
{
	"xrtDateFldMax",
	"RD_xrtFldMax",
	XmNxrtFldMax,
	"XmNxrtFldMax",
	XRTFLD_DATE,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldMax),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtDateFldResNumMax,
	FALSE,
	TRUE
},
{
	"xrtDateFldMilleniumThreshold",
	"RD_xrtFldMilleniumThreshold",
	XmNxrtFldMilleniumThreshold,
	"XmNxrtFldMilleniumThreshold",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldMilleniumThreshold),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtDateFldResNumMilleniumThreshold,
	FALSE,
	TRUE
},
{
	"xrtDateFldMin",
	"RD_xrtFldMin",
	XmNxrtFldMin,
	"XmNxrtFldMin",
	XRTFLD_DATE,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldMin),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtDateFldResNumMin,
	FALSE,
	TRUE
},
{
	"xrtDateFldPickList",
	"RD_xrtFldPickList",
	XmNxrtFldPickList,
	"XmNxrtFldPickList",
	XRTFLD_MULTI_TYPE_LIST,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldPickList),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtDateFldResNumPickList,
	FALSE,
	TRUE
},
{
	"xrtDateFldCalendar",
	"RD_xrtFldCalendar",
	XmNxrtFldCalendar,
	"XmNxrtFldCalendar",
	XRTFLD_WIDGET,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldCalendar),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtDateFldResNumCalendar,
	FALSE,
	TRUE
},
{
	"xrtDateFldCalendarCallback",
	"RD_xrtFldCalendarCallback",
	XmNxrtFldCalendarCallback,
	"XmNxrtFldCalendarCallback",
	XRTFLD_CALLBACK,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldCalendarCallback),
	XRTDIV_BEHAVIOR,
	XRTPASS0,
	NULL,
	xrtDateFldResNumCalendarCallback,
	FALSE,
	TRUE
},
{
	"xrtDateFldSpinPolicy",
	"RD_xrtFldSpinPolicy",
	XmNxrtFldSpinPolicy,
	"XmNxrtFldSpinPolicy",
	XRTFLD_SPIN_POLICY,
	{0, 0, 0},
	Offset(UxXrtDateFieldClass, XrtDateField.RD_xrtFldSpinPolicy),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtDateFldResNumSpinPolicy,
	FALSE,
	TRUE
}
};

int numflddat_res =
(sizeof(xrtflddat_res_table) / sizeof(xrtflddat_res_table[0]));

/* Resource table for XRT Float field widget */

XrtFldResourceDesc xrtfldflt_res_table[] = {
{
	"xrtFloatFldCounterIncrement",
	"RD_xrtFldCounterIncrement",
	XmNxrtFldCounterIncrement,
	"XmNxrtFldCounterIncrement",
	XRTFLD_DOUBLE,
	{0, 0, 0},
	Offset(UxXrtFloatFieldClass, XrtFloatField.RD_xrtFldCounterIncrement),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFloatFldResNumCounterIncrement,
	FALSE,
	TRUE
},
{
	"xrtFloatFldDecimalPlaces",
	"RD_xrtFldDecimalPlaces",
	XmNxrtFldDecimalPlaces,
	"XmNxrtFldDecimalPlaces",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtFloatFieldClass, XrtFloatField.RD_xrtFldDecimalPlaces),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFloatFldResNumDecimalPlaces,
	FALSE,
	TRUE
},
{
	"xrtFloatFldDefaultValue",
	"RD_xrtFldDefaultValue",
	XmNxrtFldDefaultValue,
	"XmNxrtFldDefaultValue",
	XRTFLD_DOUBLE,
	{0, 0, 0},
	Offset(UxXrtFloatFieldClass, XrtFloatField.RD_xrtFldDefaultValue),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFloatFldResNumDefaultValue,
	FALSE,
	TRUE
},
{
	"xrtFloatFldDisplayFormatString",
	"RD_xrtFldDisplayFormatString",
	XmNxrtFldDisplayFormatString,
	"XmNxrtFldDisplayFormatString",
	XRTFLD_STRING,
	{0, 0, 0},
	Offset(UxXrtFloatFieldClass, XrtFloatField.RD_xrtFldDisplayFormatString),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFloatFldResNumDisplayFormatString,
	FALSE,
	TRUE
},
{
	"xrtFloatFldInternalValue",
	"RD_xrtFldInternalValue",
	XmNxrtFldInternalValue,
	"XmNxrtFldInternalValue",
	XRTFLD_DOUBLE,
	{0, 0, 0},
	Offset(UxXrtFloatFieldClass, XrtFloatField.RD_xrtFldInternalValue),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFloatFldResNumInternalValue,
	FALSE,
	TRUE
},
{
	"xrtFloatFldLeadingZero",
	"RD_xrtFldLeadingZero",
	XmNxrtFldLeadingZero,
	"XmNxrtFldLeadingZero",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtFloatFieldClass, XrtFloatField.RD_xrtFldLeadingZero),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFloatFldResNumLeadingZero,
	FALSE,
	TRUE
},
{
	"xrtFloatFldMax",
	"RD_xrtFldMax",
	XmNxrtFldMax,
	"XmNxrtFldMax",
	XRTFLD_DOUBLE,
	{0, 0, 0},
	Offset(UxXrtFloatFieldClass, XrtFloatField.RD_xrtFldMax),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFloatFldResNumMax,
	FALSE,
	TRUE
},
{
	"xrtFloatFldMin",
	"RD_xrtFldMin",
	XmNxrtFldMin,
	"XmNxrtFldMin",
	XRTFLD_DOUBLE,
	{0, 0, 0},
	Offset(UxXrtFloatFieldClass, XrtFloatField.RD_xrtFldMin),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFloatFldResNumMin,
	FALSE,
	TRUE
},
{
	"xrtFloatFldNegativeFormat",
	"RD_xrtFldNegativeFormat",
	XmNxrtFldNegativeFormat,
	"XmNxrtFldNegativeFormat",
	XRTFLD_NEG_FORMAT,
	{0, 0, 0},
	Offset(UxXrtFloatFieldClass, XrtFloatField.RD_xrtFldNegativeFormat),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFloatFldResNumNegativeFormat,
	FALSE,
	TRUE
},
{
	"xrtFloatFldPickList",
	"RD_xrtFldPickList",
	XmNxrtFldPickList,
	"XmNxrtFldPickList",
	XRTFLD_MULTI_TYPE_LIST,
	{0, 0, 0},
	Offset(UxXrtFloatFieldClass, XrtFloatField.RD_xrtFldPickList),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFloatFldResNumPickList,
	FALSE,
	TRUE
}
};

int numfldflt_res =
(sizeof(xrtfldflt_res_table) / sizeof(xrtfldflt_res_table[0]));


/* Resource table for XRT Int field widget */

XrtFldResourceDesc xrtfldint_res_table[] = {
{
	"xrtIntFldCounterIncrement",
	"RD_xrtFldCounterIncrement",
	XmNxrtFldCounterIncrement,
	"XmNxrtFldCounterIncrement",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtIntFieldClass, XrtIntField.RD_xrtFldCounterIncrement),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtIntFldResNumCounterIncrement,
	FALSE,
	TRUE
},
{
	"xrtIntFldDefaultValue",
	"RD_xrtFldDefaultValue",
	XmNxrtFldDefaultValue,
	"XmNxrtFldDefaultValue",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtIntFieldClass, XrtIntField.RD_xrtFldDefaultValue),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtIntFldResNumDefaultValue,
	FALSE,
	TRUE
},
{
	"xrtIntFldDisplayFormatString",
	"RD_xrtFldDisplayFormatString",
	XmNxrtFldDisplayFormatString,
	"XmNxrtFldDisplayFormatString",
	XRTFLD_STRING,
	{0, 0, 0},
	Offset(UxXrtIntFieldClass, XrtIntField.RD_xrtFldDisplayFormatString),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtIntFldResNumDisplayFormatString,
	FALSE,
	TRUE
},
{
	"xrtIntFldInternalValue",
	"RD_xrtFldInternalValue",
	XmNxrtFldInternalValue,
	"XmNxrtFldInternalValue",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtIntFieldClass, XrtIntField.RD_xrtFldInternalValue),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtIntFldResNumInternalValue,
	FALSE,
	TRUE
},
{
	"xrtIntFldMax",
	"RD_xrtFldMax",
	XmNxrtFldMax,
	"XmNxrtFldMax",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtIntFieldClass, XrtIntField.RD_xrtFldMax),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtIntFldResNumMax,
	FALSE,
	TRUE
},
{
	"xrtIntFldMin",
	"RD_xrtFldMin",
	XmNxrtFldMin,
	"XmNxrtFldMin",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtIntFieldClass, XrtIntField.RD_xrtFldMin),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtIntFldResNumMin,
	FALSE,
	TRUE
},
{
	"xrtIntFldNegativeFormat",
	"RD_xrtFldNegativeFormat",
	XmNxrtFldNegativeFormat,
	"XmNxrtFldNegativeFormat",
	XRTFLD_NEG_FORMAT,
	{0, 0, 0},
	Offset(UxXrtIntFieldClass, XrtIntField.RD_xrtFldNegativeFormat),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtIntFldResNumNegativeFormat,
	FALSE,
	TRUE
},
{
	"xrtIntFldPickList",
	"RD_xrtFldPickList",
	XmNxrtFldPickList,
	"XmNxrtFldPickList",
	XRTFLD_MULTI_TYPE_LIST,
	{0, 0, 0},
	Offset(UxXrtIntFieldClass, XrtIntField.RD_xrtFldPickList),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtIntFldResNumPickList,
	FALSE,
	TRUE
}
};

int numfldint_res =
(sizeof(xrtfldint_res_table) / sizeof(xrtfldint_res_table[0]));

XrtFldResourceDesc xrtfldcmb_res_table[] = {
{
	"xrtComboArrow",
	"RD_xrtComboArrow",
	XmNxrtComboArrow,
	"XmNxrtComboArrow",
	XRTFLD_WIDGET,
	{0, 0, 0},
	Offset(UxXrtComboBoxClass, XrtComboBox.RD_xrtComboArrow),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumComboArrow,
	TRUE,
	TRUE
},
{
	"xrtComboArrowClass",
	"RD_xrtComboArrowClass",
	XmNxrtComboArrowClass,
	"XmNxrtComboArrowClass",
	XRTFLD_WIDGET_CLASS,
	{0, 0, 0},
	Offset(UxXrtComboBoxClass, XrtComboBox.RD_xrtComboArrowClass),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumComboArrowClass,
	FALSE,
	TRUE
},
{ /* pds */
	"xrtComboArrowType",
	"RD_xrtComboArrowType",
	XmNxrtComboArrowType,
	"XmNxrtComboArrowType",
	XRTFLD_ARROW_TYPE,
	{0, 0, 0},
	Offset(UxXrtComboBoxClass, XrtComboBox.RD_xrtComboArrowType),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumComboArrowType,
	FALSE,
	TRUE
},
{
	"xrtComboField",
	"RD_xrtComboField",
	XmNxrtComboField,
	"XmNxrtComboField",
	XRTFLD_WIDGET,
	{0, 0, 0},
 	Offset(UxXrtComboBoxClass, XrtComboBox.RD_xrtComboField),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumComboField,
	TRUE,
	TRUE
},
{
	"xrtComboFieldClass",
	"RD_xrtComboFieldClass",
	XmNxrtComboFieldClass,
	"XmNxrtComboFieldClass",
	XRTFLD_WIDGET_CLASS,
	{0, 0, 0},
	Offset(UxXrtComboBoxClass, XrtComboBox.RD_xrtComboFieldClass),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumComboFieldClass,
	FALSE,
	TRUE
},
{
	"xrtComboFieldType",
	"RD_xrtComboFieldType",
	XmNxrtComboFieldType,
	"XmNxrtComboFieldType",
	XRTFLD_FIELD_TYPE,
	{0, 0, 0},
	Offset(UxXrtComboBoxClass, XrtComboBox.RD_xrtComboFieldType),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumComboFieldType,
	FALSE,
	TRUE
},
{ /* pds */
	"xrtComboSpacing",
	"RD_xrtComboSpacing",
	XmNxrtComboSpacing,
	"XmNxrtComboSpacing",
	XRTFLD_SHORT,
	{0, 0, 0},
	Offset(UxXrtComboBoxClass, XrtComboBox.RD_xrtComboSpacing),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumComboSpacing,
	FALSE,
	TRUE
}
};

int numfldcmb_res =
(sizeof(xrtfldcmb_res_table) / sizeof(xrtfldcmb_res_table[0]));

XrtFldResourceDesc xrtfldspn_res_table[] = {
{
	"xrtSpinArrowLayout",
	"RD_xrtSpinArrowLayout",
	XmNxrtSpinArrowLayout,
	"XmNxrtSpinArrowLayout",
	XRTFLD_SPIN_ARROW_LAYOUT,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinArrowLayout),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinArrowLayout,
	FALSE,
	TRUE
},
{
	"xrtSpinArrowOrientation",
	"RD_xrtSpinArrowOrientation",
	XmNxrtSpinArrowOrientation,
	"XmNxrtSpinArrowOrientation",
	XRTFLD_SPIN_ARROW_ORIENTATION,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinArrowOrientation),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinArrowOrientation,
	FALSE,
	TRUE
},
{ /* pds */
	"xrtSpinArrowType",
	"RD_xrtSpinArrowType",
	XmNxrtSpinArrowType,
	"XmNxrtSpinArrowType",
	XRTFLD_ARROW_TYPE,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinArrowType),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinArrowType,
	FALSE,
	TRUE
},
{
	"xrtSpinDecrArrow",
	"RD_xrtSpinDecrArrow",
	XmNxrtSpinDecrArrow,
	"XmNxrtSpinDecrArrow",
	XRTFLD_WIDGET,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinDecrArrow),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinDecrArrow,
	TRUE,
	TRUE
},
{
	"xrtSpinDecrArrowClass",
	"RD_xrtSpinDecrArrowClass",
	XmNxrtSpinDecrArrowClass,
	"XmNxrtSpinDecrArrowClass",
	XRTFLD_WIDGET_CLASS,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinDecrArrowClass),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinDecrArrowClass,
	FALSE,
	TRUE
},
{
	"xrtSpinField",
	"RD_xrtSpinField",
	XmNxrtSpinField,
	"XmNxrtSpinField",
	XRTFLD_WIDGET,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinField),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinField,
	TRUE,
	TRUE
},
{
	"xrtSpinFieldClass",
	"RD_xrtSpinFieldClass",
	XmNxrtSpinFieldClass,
	"XmNxrtSpinFieldClass",
	XRTFLD_WIDGET_CLASS,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinFieldClass),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinFieldClass,
	TRUE,
	TRUE
},
{
	"xrtSpinFieldName",
	"RD_xrtSpinFieldName",
	XmNxrtSpinFieldName,
	"XmNxrtSpinFieldName",
	XRTFLD_STRING,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinFieldName),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinFieldName,
	TRUE,
	TRUE
},
{
	"xrtSpinFieldType",
	"RD_xrtSpinFieldType",
	XmNxrtSpinFieldType,
	"XmNxrtSpinFieldType",
	XRTFLD_FIELD_TYPE,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinFieldType),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinFieldType,
	FALSE,
	TRUE
},
{
	"xrtSpinIncrArrow",
	"RD_xrtSpinIncrArrow",
	XmNxrtSpinIncrArrow,
	"XmNxrtSpinIncrArrow",
	XRTFLD_WIDGET,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinIncrArrow),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinIncrArrow,
	TRUE,
	TRUE
},
{
	"xrtSpinIncrArrowClass",
	"RD_xrtSpinIncrArrowClass",
	XmNxrtSpinIncrArrowClass,
	"XmNxrtSpinIncrArrowClass",
	XRTFLD_WIDGET_CLASS,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinIncrArrowClass),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinIncrArrowClass,
	FALSE,
	TRUE
},
{ /* pds */
	"xrtSpinSpacing",
	"RD_xrtSpinSpacing",
	XmNxrtSpinSpacing,
	"XmNxrtSpinSpacing",
	XRTFLD_SHORT,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinSpacing),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinSpacing,
	FALSE,
	TRUE
},
{
	"xrtSpinType",
	"RD_xrtSpinType",
	XmNxrtSpinType,
	"XmNxrtSpinType",
	XRTFLD_SPIN_TYPE,
	{0, 0, 0},
	Offset(UxXrtSpinBoxClass, XrtSpinBox.RD_xrtSpinType),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtFldResNumSpinType,
	FALSE,
	TRUE
}
};

int numfldspn_res =
(sizeof(xrtfldspn_res_table) / sizeof(xrtfldspn_res_table[0]));

XrtFldResourceDesc xrtfldcal_res_table[] = {
{
	"xrtCalendarCellMargin",
	"RD_xrtCalendarCellMargin",
	XmNxrtCalendarCellMargin,
	"XmNxrtCalendarCellMargin",
	XRTFLD_SHORT, 
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarCellMargin),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumCellMargin,
	FALSE,
	TRUE
},
{
	"xrtCalendarCellHighlightColor",
	"RD_xrtCalendarCellHighlightColor",
	XmNxrtCalendarCellHighlightColor,
	"XmNxrtCalendarCellHighlightColor",
	XRTFLD_COLOR,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarCellHighlightColor),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumCellHighlightColor,
	FALSE,
	TRUE
},
{
	"xrtCalendarCellHighlightThickness",
	"RD_xrtCalendarCellHighlightThickness",
	XmNxrtCalendarCellHighlightThickness,
	"XmNxrtCalendarCellHighlightThickness",
	XRTFLD_SHORT, 
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarCellHighlightThickness),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumCellHighlightThickness,
	FALSE,
	TRUE
},
{
	"xrtCalendarCreateMonth",
	"RD_xrtCalendarCreateMonth",
	XmNxrtCalendarCreateMonth,
	"XmNxrtCalendarCreateMonth",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarCreateMonth),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumCreateMonth,
	FALSE,
	TRUE
},
{
	"xrtCalendarCreateTime",
	"RD_xrtCalendarCreateTime",
	XmNxrtCalendarCreateTime,
	"XmNxrtCalendarCreateTime",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarCreateTime),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumCreateTime,
	FALSE,
	TRUE
},
{
	"xrtCalendarCreateYear",
	"RD_xrtCalendarCreateYear",
	XmNxrtCalendarCreateYear,
	"XmNxrtCalendarCreateYear",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarCreateYear),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumCreateYear,
	FALSE,
	TRUE
},
{
	"xrtCalendarEnabled",
	"RD_xrtCalendarEnabled",
	XmNxrtCalendarEnabled,
	"XmNxrtCalendarEnabled",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarEnabled),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumEnabled,
	FALSE,
	TRUE
},
{
	"xrtCalendarFirstDayOfWeek",
	"RD_xrtCalendarFirstDayOfWeek",
	XmNxrtCalendarFirstDayOfWeek,
	"XmNxrtCalendarFirstDayOfWeek",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarFirstDayOfWeek),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumFirstDayOfWeek,
	FALSE,
	TRUE
},
{
	"xrtCalendarFullWeekdayNames",
	"RD_xrtCalendarFullWeekdayNames",
	XmNxrtCalendarFullWeekdayNames,
	"XmNxrtCalendarFullWeekdayNames",
	XRTFLD_BOOLEAN,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarFullWeekdayNames),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumFullWeekdayNames,
	FALSE,
	TRUE
},
{
	"xrtCalendarHeaderNumChars",
	"RD_xrtCalendarHeaderNumChars",
	XmNxrtCalendarHeaderNumChars,
	"XmNxrtCalendarHeaderNumChars",
	XRTFLD_INT,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarHeaderNumChars),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumHeaderNumChars,
	FALSE,
	TRUE
},
{
	"xrtCalendarHeaderBackground",
	"RD_xrtCalendarHeaderBackground",
	XmNxrtCalendarHeaderBackground,
	"XmNxrtCalendarHeaderBackground",
	XRTFLD_COLOR,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarHeaderBackground),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumHeaderBackground,
	FALSE,
	TRUE
},
{
	"xrtCalendarHeaderFontList",
	"RD_xrtCalendarHeaderFontList",
	XmNxrtCalendarHeaderFontList,
	"XmNxrtCalendarHeaderFontList",
	XRTFLD_FONT_LIST,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarHeaderFontList),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumHeaderFontList,
	FALSE,
	TRUE
},
{
	"xrtCalendarHeaderForeground",
	"RD_xrtCalendarHeaderForeground",
	XmNxrtCalendarHeaderForeground,
	"XmNxrtCalendarHeaderForeground",
	XRTFLD_COLOR,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarHeaderForeground),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumHeaderForeground,
	FALSE,
	TRUE
},
{
	"xrtCalendarMarginBottom",
	"RD_xrtCalendarMarginBottom",
	XmNxrtCalendarMarginBottom,
	"XmNxrtCalendarMarginBottom",
	XRTFLD_SHORT, 
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarMarginBottom),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumMarginBottom,
	FALSE,
	TRUE
},
{
	"xrtCalendarMarginLeft",
	"RD_xrtCalendarMarginLeft",
	XmNxrtCalendarMarginLeft,
	"XmNxrtCalendarMarginLeft",
	XRTFLD_SHORT, 
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarMarginLeft),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumMarginLeft,
	FALSE,
	TRUE
},
{
	"xrtCalendarMarginRight",
	"RD_xrtCalendarMarginRight",
	XmNxrtCalendarMarginRight,
	"XmNxrtCalendarMarginRight",
	XRTFLD_SHORT, 
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarMarginRight),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumMarginRight,
	FALSE,
	TRUE
},
{
	"xrtCalendarMarginTop",
	"RD_xrtCalendarMarginTop",
	XmNxrtCalendarMarginTop,
	"XmNxrtCalendarMarginTop",
	XRTFLD_SHORT, 
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarMarginTop),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumMarginTop,
	FALSE,
	TRUE
},
{
	"xrtCalendarMaxRollDate",
	"RD_xrtCalendarMaxRollDate",
	XmNxrtCalendarMaxRollDate,
	"XmNxrtCalendarMaxRollDate",
	XRTFLD_DATE,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarMaxRollDate),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumMaxRollDate,
	FALSE,
	TRUE
},
{
	"xrtCalendarMinRollDate",
	"RD_xrtCalendarMinRollDate",
	XmNxrtCalendarMinRollDate,
	"XmNxrtCalendarMinRollDate",
	XRTFLD_DATE,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarMinRollDate),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumMinRollDate,
	FALSE,
	TRUE
},
{
	"xrtCalendarModifyCallback",
	"RD_xrtCalendarModifyCallback",
	XmNxrtCalendarModifyCallback,
	"XmNxrtCalendarModifyCallback",
	XRTFLD_CALLBACK,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarModifyCallback ),
	XRTDIV_BEHAVIOR,
	XRTPASS0,
	NULL,
	xrtCalendarResNumModifyCallback,
	FALSE,
	TRUE
},
{
	"xrtCalendarSpecialDayBackground",
	"RD_xrtCalendarSpecialDayBackground",
	XmNxrtCalendarSpecialDayBackground,
	"XmNxrtCalendarSpecialDayBackground",
	XRTFLD_COLOR,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarSpecialDayBackground),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumSpecialDayBackground,
	FALSE,
	TRUE
},
{
	"xrtCalendarSpecialDayForeground",
	"RD_xrtCalendarSpecialDayForeground",
	XmNxrtCalendarSpecialDayForeground,
	"XmNxrtCalendarSpecialDayForeground",
	XRTFLD_COLOR,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarSpecialDayForeground),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumSpecialDayForeground,
	FALSE,
	TRUE
},
{
	"xrtCalendarSpecialDays",
	"RD_xrtCalendarSpecialDays",
	XmNxrtCalendarSpecialDays,
	"XmNxrtCalendarSpecialDays",
	XRTFLD_SPECIAL_DAYS,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarSpecialDays),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumSpecialDays,
	FALSE,
	TRUE
},
{
	"xrtCalendarSelectedBackground",
	"RD_xrtCalendarSelectedBackground",
	XmNxrtCalendarSelectedBackground,
	"XmNxrtCalendarSelectedBackground",
	XRTFLD_COLOR,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarSelectedBackground),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumSelectedBackground,
	FALSE,
	TRUE
},
{
	"xrtCalendarSelectedForeground",
	"RD_xrtCalendarSelectedForeground",
	XmNxrtCalendarSelectedForeground,
	"XmNxrtCalendarSelectedForeground",
	XRTFLD_COLOR,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarSelectedForeground),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumSelectedForeground,
	FALSE,
	TRUE
},
{
	"xrtCalendarValue",
	"RD_xrtCalendarValue",
	XmNxrtCalendarValue,
	"XmNxrtCalendarValue",
	XRTFLD_DATE,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarValue),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumValue,
	FALSE,
	TRUE
},
{
	"xrtCalendarValueLong",
	"RD_xrtCalendarValueLong",
	XmNxrtCalendarValueLong,
	"XmNxrtCalendarValueLong",
	XRTFLD_LONG,
	{0, 0, 0},
	Offset(UxXrtCalendarClass, XrtCalendar.RD_xrtCalendarValueLong),
	XRTDIV_SPECIFIC,
	XRTPASS0,
	NULL,
	xrtCalendarResNumValueLong,
	FALSE,
	TRUE
}
};

int numfldcal_res =
(sizeof(xrtfldcal_res_table) / sizeof(xrtfldcal_res_table[0]));


XrtResourceType **xrtfld_res_types = NULL;
int				numfld_res_types;

void
XrtFld_init_resource_types()
{
	XrtResourceType	*type;
	int				n;
	static Boolean	AlreadyDone = FALSE;
	
	if (AlreadyDone) {
		return;
	}

	n = 0;
	xrtfld_res_types = (XrtResourceType **)
		calloc(n+2, sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_ARROW_TYPE;  /* pds */
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldArrowType;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldArrowType;
	type->validate = UxValidateXrtFldArrowType;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_BOOLEAN;
	type->utype = UxUT_string;
	type->xtype = UxXT_Boolean;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfBoolean;
	type->validate = UxValidateBoolean;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_CALLBACK;
	type->utype = (short) 0;
	type->xtype = 0;
	type->convert_flag = 0;
	type->converter = NULL;
	type->values_of = NULL;
	type->validate = NULL;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_CASE;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldCase;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldCase;
	type->validate = UxValidateXrtFldCase;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_COLOR;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldPixel;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldPixel;
	type->validate = UxValidateString;
	type->edit_func = UxGUIPopupColorView;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_DATE;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldDate;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldDate;
	type->validate = UxValidateXrtFldDate;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_DATE_INCREMENT;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldDateIncrement;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldDateIncrement;
	type->validate = UxValidateString;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_DISP_SCROLLBAR;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldDispScrollBar;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldDispScrollBar;
	type->validate = UxValidateXrtFldDispScrollBar;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_DOUBLE;
	type->utype = UxUT_float;
	type->xtype = UxXT_XrtFldDouble;
	type->convert_flag = 0;
	type->converter = "UxConvertXrtFldDouble";
	type->values_of = UxValuesOfXrtFldDouble;
	type->validate = UxValidateInt;
	type->edit_func = NULL;
	type->get_func = (int (*)())XrtFldGetDouble;
	type->put_func = XrtFldPutDouble;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_ERROR;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldError;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldError;
	type->validate = UxValidateXrtFldError;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_FIELD_TYPE;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldFieldType;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldFieldType;
	type->validate = UxValidateXrtFldFieldType;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_FORMAT;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldFormat;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldFormat;
	type->validate = UxValidateXrtFldFormat;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_INT;
	type->utype = UxUT_int;
	type->xtype = UxXT_int;
	type->convert_flag = 0;
	type->converter = NULL;
	type->values_of = UxValuesOfInt;
	type->validate = UxValidateInt;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_LONG;
	type->utype = UxUT_long;
	type->xtype = UxXT_int;
	type->convert_flag = 1;
	type->converter = "UxConvertXrtFldLong";
	type->values_of = UxValuesOfInt;
	type->validate = UxValidateInt;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_MATCH_MASK;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldMatchMask;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldMatchMask;
	type->validate = UxValidateXrtFldMatchMask;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_MENU_ALIGNMENT;  /* pds */
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldMenuAlignment;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldMenuAlignment;
	type->validate = UxValidateXrtFldMenuAlignment;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_MULTI_TYPE_LIST;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldMultiTypeList;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldMultiTypeList;
	type->validate = UxValidateXrtFldMultiTypeList;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_NEG_FORMAT;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldNegFormat;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldNegFormat;
	type->validate = UxValidateXrtFldNegFormat;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_RANGE;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldRange;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldRange;
	type->validate = UxValidateXrtFldRange;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;

	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_SEARCH_POLICY; /* pds */
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldSearchPolicy;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldSearchPolicy;
	type->validate = UxValidateXrtFldSearchPolicy;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	  
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_SHORT;
	type->utype = UxUT_short;
	type->xtype = UxXT_short;
	type->convert_flag = 0;
	type->converter = NULL;
	type->values_of = UxValuesOfInt;
	type->validate = UxValidateInt;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;

	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_SPIN_ARROW_LAYOUT;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldSpinArrowLayout;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldSpinArrowLayout;
	type->validate = UxValidateXrtFldSpinArrowLayout;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_SPIN_ARROW_ORIENTATION;
	type->utype = UxUT_string;
	type->xtype = UxXT_Orientation;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfOrientation;
	type->validate = UxValidateOrientation;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_SPIN_TYPE;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldSpinType;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldSpinType;
	type->validate = UxValidateXrtFldSpinType;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_STRING;
	type->utype = UxUT_string;
	type->xtype = UxXT_String;
	type->convert_flag = 0;
	type->converter = NULL;
	type->values_of = UxValuesOfString;
	type->validate = UxValidateString;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_STRING_TABLE;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldStringTable;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldStringTable;
	type->validate = UxValidateString;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_WIDGET;
	type->utype = UxUT_string;
	type->xtype = UxXT_Widget;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfWidget;
	type->validate = UxValidateWidget;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
 	xrtfld_res_types[n++] = type;

	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_WIDGET_CLASS;
	type->utype = UxUT_string;
	type->xtype = UxXT_WidgetClass;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfString;
	type->validate = UxValidateString;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
 	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_XMSTRING;
	type->utype = UxUT_string;
	type->xtype = UxXT_XmString;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXmString;
	type->validate = UxValidateXmString;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;
	
	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_POPUP_TYPE;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldPopupType;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldPopupType;
	type->validate = UxValidateXrtFldPopupType;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;

	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_FONT_LIST;
	type->utype = UxUT_string;
	type->xtype = UxXT_XmFontList;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXmFontList;
	type->validate = UxValidateXmFontList;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;

	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_SPECIAL_DAYS;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldSpecialDays;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldSpecialDays;
	type->validate = UxValidateString;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;

	xrtfld_res_types = (XrtResourceType **)
		realloc(xrtfld_res_types, (n+2) * sizeof(XrtResourceType *));
	type = (XrtResourceType *) calloc (1, sizeof(XrtResourceType));
	type->datatype.type_field = XRTFLD_SPIN_POLICY;
	type->utype = UxUT_string;
	type->xtype = UxXT_XrtFldSpinPolicy;
	type->convert_flag = 1;
	type->converter = NULL;
	type->values_of = UxValuesOfXrtFldSpinPolicy;
	type->validate = UxValidateXrtFldSpinPolicy;
	type->edit_func = NULL;
	type->get_func = NULL;
	type->put_func = NULL;
	xrtfld_res_types[n++] = type;

 	xrtfld_res_types[n] = NULL;
	numfld_res_types = n;
	AlreadyDone = TRUE;

}

static void
XrtFld_initialize_resources(table, nres, nres_string)
XrtFldResourceDesc	*table;
int					nres;
char				*nres_string;
{
	int		itype, n;
	char	buffer[128];
	
	sprintf(buffer, "%s", nres_string);
	
	for (n = 0; n < nres; n++) {
		itype = (int) table[n].res_type;
		table[n].type = xrtfld_res_types[itype];
		if (table[n].resnum != n) {
			sprintf(buffer,
					"Field resource number (%d) & table number (%d) unequal!",
					table[n].resnum, n);
			printf("%s\n", buffer);
		}
	}

}

void
XrtFld_init_resource_table()
{
	static Boolean	AlreadyDone = FALSE;

	if (AlreadyDone) {
		return;
	}

	XrtFld_init_resource_types();

	XrtFld_initialize_resources(xrtfld_res_table, numfld_res, "numfld_res");
	XrtFld_initialize_resources(xrtfldstr_res_table, numfldstr_res,
							    "numfldstr_res");
	XrtFld_initialize_resources(xrtflddat_res_table, numflddat_res,
							    "numflddat_res");
	XrtFld_initialize_resources(xrtfldflt_res_table, numfldflt_res,
							    "numfldflt_res");
	XrtFld_initialize_resources(xrtfldint_res_table, numfldint_res,
							    "numfldint_res");
	XrtFld_initialize_resources(xrtfldcmb_res_table, numfldcmb_res,
							    "numfldcmb_res");
	XrtFld_initialize_resources(xrtfldspn_res_table, numfldspn_res,
							    "numfldspn_res");
	XrtFld_initialize_resources(xrtfldcal_res_table, numfldcal_res,
							    "numfldcal_res");
	AlreadyDone = TRUE;

/* for yen mark problem */
#ifdef hp700v9
	{
		char *lang;
		char *lc_monetary;

		lang=getenv("LANG");
		lc_monetary=getenv("LC_MONETARY");
		if ( lc_monetary!=NULL || (lang!=NULL && strcmp(lang,"C")!=0)){
			XrtFldSetDefaultLocaleStruct(localeconv(),NULL);
		}
	}
#endif /* hp700v9 */


}

#ifdef DESIGN_TIME

static float
XrtFldGetDouble(sw, res)
swidget         sw;
Resource_t      *res;
{
	Widget		w;
	double		value = 0.0;
	char		*ptr;
	XtArgVal	argval;

	ptr = UxGetXtName(res);
	if (UxIsSwidget(sw))
		if ((w = UxGetWidget(sw)) != NULL)
			XtVaGetValues (w, ptr, &value, NULL);
		else
			if(UxFindValue(sw, res, &argval))
				value = XrtFldArgValToDouble(argval);

#ifdef osf1
#include <limits.h>
	if (value > FLT_MAX) {
		value = FLT_MAX;
	}
	if (value < -FLT_MAX) {
		value = -FLT_MAX;
	}
#endif /* osf1 */
	return (float)value;
}

static int
XrtFldPutDouble(sw, res, intvalue)
swidget		sw;
Resource_t	*res;
int		intvalue;
{
	Widget		w;
	char		*ptr;
	int		status = ERROR;
	XtArgVal	argval;
	FloatInt_t	localcast;
	float		value;

	localcast.i = intvalue;
	value = localcast.f;

	ptr = UxGetXtName(res);

	if (UxIsSwidget(sw)) {
		argval = XrtFldDoubleToArgVal((double)value);
		UxSaveValue (sw, res, argval, FALSE);

		/*
		 *  If the delay flag is NOT set, update widget now
		 *  so that the change happens immediately in the
		 *  property editor.
		 */
		if(!UxGetDelay(sw) && (w = (Widget) UxGetWidget(sw)) != NULL) {
			XtVaSetValues(w, ptr, argval, NULL);
		}

		status = NO_ERROR;
	}
	else {
		status = ERROR;
	}
	
	return(status);
}

#else

static float XrtFldGetDouble() {}
static int   XrtFldPutDouble() {}

#endif /* DESIGN_TIME */
