/* $AstecId: XrtGph_xtype.c,v 1.4 1999/10/13 11:48:28 nakayama Exp $ */
/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/graph widget UIM/X integration file
 *
 *  SCCSID -- @(#)XrtGph_xtype.c	1.12    99/01/29 KL Group Inc.
 */

/******************************************************************************
				XrtGph_xtype.c
******************************************************************************/

#include <UxLib.h>
#include <vtypes.h>
#include <utype.h>
#include <ctype.h>
#include <string.h>

#ifdef DESIGN_TIME

#include "veos.h"       		/* Visual Edge object system */
#include "valuesOf.h"   		/* ValuesOf & Validator functions */
#include "resource.h"
#include "resvalue.h"
#include "plist.cl.h"
#include "observer.h"

#endif /* DESIGN_TIME */

#include "Xrt_restable.h"

#include <Xm/XrtGraph.h>

#include <cat_macros.h>

/* Begin new XRT/graph types */

int		UxXT_float;
int		UxXT_XrtGphAdjust;
int		UxXT_XrtGphAlign;
int		UxXT_XrtGphAnchor;
int		UxXT_XrtGphAngleUnit;		/*graph 3.0*/
int		UxXT_XrtGphAnnotationMethod;
int		UxXT_XrtGphAnnoPlacement;
int		UxXT_XrtGphBorder;
int		UxXT_XrtGphData;
int		UxXT_XrtGphData2;
int		UxXT_XrtGphDataStyle;
int		UxXT_XrtGphDataStyles;
int		UxXT_XrtGphMarkerMethod;	/*graph 3.0*/
int		UxXT_XrtGphNumMethod;
int		UxXT_XrtGphOriginPlacement;
int		UxXT_XrtGphPieOrder;
int		UxXT_XrtGphPieThresholdMethod;
int		UxXT_XrtGphRotate;
int		UxXT_XrtGphShading;			/*graph 3.0*/
int		UxXT_XrtGphStrings;
int		UxXT_XrtGphTimeUnit;
int		UxXT_XrtGphType;
int		UxXT_XrtGphValueLabels;

int     UxXT_XrtGphBubbleMethod;
int     UxXT_XrtGphImage;
int     UxXT_XrtGphImageLayout;

/* Define three static arrays for each enumerated type */

/* Adjust */
static char *uXrtGphAdjust[] = {
	"AdjustLeft",
	"AdjustRight",
	"AdjustCenter"
};
static int xXrtGphAdjust[] = {
	XRT_ADJUST_LEFT,
	XRT_ADJUST_RIGHT,
	XRT_ADJUST_CENTER
};
static char	*dXrtGphAdjust[] = {
	"XRT_ADJUST_LEFT",
	"XRT_ADJUST_RIGHT",
	"XRT_ADJUST_CENTER"
};

/* Align */
static char *uXrtGphAlign[] = {
	"AlignVertical",
	"AlignHorizontal"
};
static int xXrtGphAlign[] = {
	XRT_ALIGN_VERTICAL,
	XRT_ALIGN_HORIZONTAL
};
static char *dXrtGphAlign[] = {
	"XRT_ALIGN_VERTICAL",
	"XRT_ALIGN_HORIZONTAL"
};

/* Anchor */
static char *uXrtGphAnchor[] = {
	"AnchorNorth",
	"AnchorSouth", 
	"AnchorEast",
	"AnchorWest", 
	"AnchorNortheast",
	"AnchorNorthwest", 
	"AnchorSoutheast",
	"AnchorSouthwest", 
	"AnchorHome",
	"AnchorBest"
};
static int xXrtGphAnchor[] = {
	XRT_ANCHOR_NORTH,
	XRT_ANCHOR_SOUTH,
	XRT_ANCHOR_EAST,
	XRT_ANCHOR_WEST, 
	XRT_ANCHOR_NORTHEAST,
	XRT_ANCHOR_NORTHWEST, 
	XRT_ANCHOR_SOUTHEAST,
	XRT_ANCHOR_SOUTHWEST, 
	XRT_ANCHOR_HOME,
	XRT_ANCHOR_BEST
};
static char *dXrtGphAnchor[] = {
	"XRT_ANCHOR_NORTH",
	"XRT_ANCHOR_SOUTH", 
	"XRT_ANCHOR_EAST",
	"XRT_ANCHOR_WEST", 
	"XRT_ANCHOR_NORTHEAST",
	"XRT_ANCHOR_NORTHWEST", 
	"XRT_ANCHOR_SOUTHEAST",
	"XRT_ANCHOR_SOUTHWEST", 
	"XRT_ANCHOR_HOME",
	"XRT_ANCHOR_BEST"
};

/* AngleUnit */
static char *uXrtGphAngleUnit[] = {
	"AngleDegrees",
    "AngleRadians",
    "AngleGrads"
};
static int xXrtGphAngleUnit[] = {
	XRT_ANGLE_DEGREES,
    XRT_ANGLE_RADIANS,
    XRT_ANGLE_GRADS
};
static char	*dXrtGphAngleUnit[] = {
	"XRT_ANGLE_DEGREES",
    "XRT_ANGLE_RADIANS",
    "XRT_ANGLE_GRADS"
};

/* AnnotationMethod */
static char *uXrtGphAnnotationMethod[] = {
	"AnnoValues",
	"AnnoPointLabels", 
	"AnnoValueLabels",
	"AnnoTimeLabels"
};
static int xXrtGphAnnotationMethod[] = {
	XRT_ANNO_VALUES,
	XRT_ANNO_POINT_LABELS, 
	XRT_ANNO_VALUE_LABELS,
	XRT_ANNO_TIME_LABELS
};
static char *dXrtGphAnnotationMethod[] = {
	"XRT_ANNO_VALUES",
	"XRT_ANNO_POINT_LABELS", 
	"XRT_ANNO_VALUE_LABELS",
	"XRT_ANNO_TIME_LABELS"
};

/* AnnoPlacement */
static char *uXrtGphAnnoPlacement[] = {
	"AnnoAuto",
	"AnnoOrigin", 
	"AnnoMin",
	"AnnoMax"
};
static int xXrtGphAnnoPlacement[] = {
	XRT_ANNO_AUTO,
	XRT_ANNO_ORIGIN, 
	XRT_ANNO_MIN,
	XRT_ANNO_MAX
};
static char *dXrtGphAnnoPlacement[] = {
	"XRT_ANNO_AUTO",
	"XRT_ANNO_ORIGIN", 
	"XRT_ANNO_MIN",
	"XRT_ANNO_MAX"
};

/* Border */
static char *uXrtGphBorder[] = {
	"BorderNone",
	"Border3dOut",
	"Border3dIn", 
	"BorderShadow",
	"BorderPlain",
	"BorderEtchedIn",
	"BorderEtchedOut",
	"BorderFrameIn",
	"BorderFrameOut",
	"BorderBevel"
};
static int xXrtGphBorder[] = {
	XRT_BORDER_NONE,
	XRT_BORDER_3D_OUT,
	XRT_BORDER_3D_IN, 
	XRT_BORDER_SHADOW,
	XRT_BORDER_PLAIN,
	XRT_BORDER_ETCHED_IN,
	XRT_BORDER_ETCHED_OUT,
	XRT_BORDER_FRAME_IN,
	XRT_BORDER_FRAME_OUT,
	XRT_BORDER_BEVEL
};
static char *dXrtGphBorder[] = {
	"XRT_BORDER_NONE",
	"XRT_BORDER_3D_OUT",
	"XRT_BORDER_3D_IN", 
	"XRT_BORDER_SHADOW",
	"XRT_BORDER_PLAIN",
	"XRT_BORDER_ETCHED_IN",
	"XRT_BORDER_ETCHED_OUT",
	"XRT_BORDER_FRAME_IN",
	"XRT_BORDER_FRAME_OUT",
	"XRT_BORDER_BEVEL"
};

/* MarkerMethod */
static char *uXrtGphMarkerMethod[] = {
	"MarkerSetPoint",
	"MarkerValue"
};
static int xXrtGphMarkerMethod[] = {
	XRT_MARKER_SET_POINT,
	XRT_MARKER_VALUE
};
static char	*dXrtGphMarkerMethod[] = {
	"XRT_MARKER_SET_POINT",
	"XRT_MARKER_VALUE"
};

/* NumMethod */
static char *uXrtGphNumMethod[] = {
	"NumPrecision",
	"NumRound"
};
static int xXrtGphNumMethod[] = {
	XRT_NUM_PRECISION,
	XRT_NUM_ROUND
};
static char *dXrtGphNumMethod[] = {
	"XRT_NUM_PRECISION",
	"XRT_NUM_ROUND"
};

/* OriginPlacement */
static char *uXrtGphOriginPlacement[] = {
	"OriginAuto",
	"OriginZero", 
	"OriginMin",
	"OriginMax"
};
static int xXrtGphOriginPlacement[] = {
	XRT_ORIGIN_AUTO,
	XRT_ORIGIN_ZERO, 
	XRT_ORIGIN_MIN,
	XRT_ORIGIN_MAX
};
static char *dXrtGphOriginPlacement[] = {
	"XRT_ORIGIN_AUTO",
	"XRT_ORIGIN_ZERO", 
	"XRT_ORIGIN_MIN",
	"XRT_ORIGIN_MAX"
};

/* PieOrder */
static char *uXrtGphPieOrder[] = {
	"PieorderAscending",
	"PieorderDescending", 
	"PieorderDataOrder"
};
static int xXrtGphPieOrder[] = {
	XRT_PIEORDER_ASCENDING,
	XRT_PIEORDER_DESCENDING,
	XRT_PIEORDER_DATA_ORDER
};
static char *dXrtGphPieOrder[] = {
	"XRT_PIEORDER_ASCENDING",
	"XRT_PIEORDER_DESCENDING", 
	"XRT_PIEORDER_DATA_ORDER"
};

/* PieThresholdMethod */
static char *uXrtGphPieThresholdMethod[] = {
	"PieSliceCutoff",
	"PiePercentile"
};
static int xXrtGphPieThresholdMethod[] = {
	XRT_PIE_SLICE_CUTOFF,
	XRT_PIE_PERCENTILE
};
static char *dXrtGphPieThresholdMethod[] = {
	"XRT_PIE_SLICE_CUTOFF",
	"XRT_PIE_PERCENTILE"
};

/* Rotate */
static char *uXrtGphRotate[] = {
	"RotateNone",
	"Rotate90",
	"Rotate270"
};
static int xXrtGphRotate[] = {
	XRT_ROTATE_NONE,
	XRT_ROTATE_90,
	XRT_ROTATE_270
};
static char *dXrtGphRotate[] = {
	"XRT_ROTATE_NONE",
	"XRT_ROTATE_90",
	"XRT_ROTATE_270"
};

/* Shading */
static char *uXrtGphShading[] = {
	"ShadingColor",
	"ShadingDithered",
	"ShadingNone"
};
static int xXrtGphShading[] = {
    XRT_SHADING_COLOR,
    XRT_SHADING_DITHERED,
    XRT_SHADING_NONE
};
static char	*dXrtGphShading[] = {
	"XRT_SHADING_COLOR",
	"XRT_SHADING_DITHERED",
	"XRT_SHADING_NONE"
};

/* TimeUnit */
static char *uXrtGphTimeUnit[] = {
	"TmunitSeconds",
	"TmunitMinutes",
	"TmunitHours", 
	"TmunitDays",
	"TmunitWeeks",
	"TmunitMonths", 
	"TmunitYears"
};
static int xXrtGphTimeUnit[] = {
	XRT_TMUNIT_SECONDS,
	XRT_TMUNIT_MINUTES,
	XRT_TMUNIT_HOURS, 
	XRT_TMUNIT_DAYS,
	XRT_TMUNIT_WEEKS,
	XRT_TMUNIT_MONTHS,
	XRT_TMUNIT_YEARS
};
static char *dXrtGphTimeUnit[] = {
	"XRT_TMUNIT_SECONDS",
	"XRT_TMUNIT_MINUTES",
	"XRT_TMUNIT_HOURS", 
	"XRT_TMUNIT_DAYS",
	"XRT_TMUNIT_WEEKS",
	"XRT_TMUNIT_MONTHS", 
	"XRT_TMUNIT_YEARS"
};

/* Type */
static char *uXrtGphType[] = {
	"TypePlot",
	"TypeBar",
	"TypePie", 
	"TypeStackingBar",
	"TypeArea",
	"TypeHilo",
	"TypeHiloOpenClose",
	"TypeCandle",
	"TypePolar",
	"TypeRadar",
	"TypeFilledRadar",
	"TypeBubble"
};
static int xXrtGphType[] = {
	XRT_TYPE_PLOT,
	XRT_TYPE_BAR,
	XRT_TYPE_PIE, 
	XRT_TYPE_STACKING_BAR,
	XRT_TYPE_AREA,
	XRT_TYPE_HILO,
	XRT_TYPE_HILO_OPEN_CLOSE,
	XRT_TYPE_CANDLE,
	XRT_TYPE_POLAR,
	XRT_TYPE_RADAR,
	XRT_TYPE_FILLED_RADAR,
    XRT_TYPE_BUBBLE	
};
static char *dXrtGphType[] = {
	"XRT_TYPE_PLOT",
	"XRT_TYPE_BAR",
	"XRT_TYPE_PIE", 
	"XRT_TYPE_STACKING_BAR",
	"XRT_TYPE_AREA",
	"XRT_TYPE_HILO",
	"XRT_TYPE_HILO_OPEN_CLOSE",
	"XRT_TYPE_CANDLE",
	"XRT_TYPE_POLAR",
	"XRT_TYPE_RADAR",
	"XRT_TYPE_FILLED_RADAR",
	"XRT_TYPE_BUBBLE"
};

static char * uXrtGphBubbleMethod[] = {
    "BubblemethodDiameter",
    "BubblemethodArea"
};
static int xXrtGphBubbleMethod[] = {
    XRT_BUBBLEMETHOD_DIAMETER,
    XRT_BUBBLEMETHOD_AREA
};
static char * dXrtGphBubbleMethod[] = {
    "XRT_BUBBLEMETHOD_DIAMETER",
    "XRT_BUBBLEMETHOD_AREA"
};

static char * uXrtGphImageLayout[] = {
    "ImagelayoutCentered",
    "ImagelayoutTiled"
};
static int xXrtGphImageLayout[] = {
    XRT_IMAGELAYOUT_CENTERED,
    XRT_IMAGELAYOUT_TILED
};
static char * dXrtGphImageLayout[] = {
    "XRT_IMAGELAYOUT_CENTERED",
    "XRT_IMAGELAYOUT_TILED"
};

#ifndef   RUNTIME

/*
 * ValuesOf functions for both enumerated types and other Xtypes
 */

int
UxValuesOfFloat(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_GPHFLOAT);
	*s = values;
	*n = 1;

	return 0;
}

int
UxValuesOfXrtGphAdjust(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphAdjust;
	*n = XtNumber(uXrtGphAdjust);
	return( *n);
}

int
UxValuesOfXrtGphAlign(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphAlign;
	*n = XtNumber(uXrtGphAlign);
	return( *n);
}

int
UxValuesOfXrtGphAnchor(s, n)
char	***s;
int		*n;
{
	*s = uXrtGphAnchor;
	*n = XtNumber(uXrtGphAnchor);
	return(*n);
}

int
UxValuesOfXrtGphAngleUnit(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphAngleUnit;
	*n = XtNumber(uXrtGphAngleUnit);
	return( *n);
}

int
UxValuesOfXrtGphBorder(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphBorder;
	*n = XtNumber(uXrtGphBorder);
	return( *n);
}

int
UxValuesOfXrtGphData(s, n)
char	***s;
int		*n;
{
	static char *values[1];
	
	values[0] = CGETS_KL(VALOF_GPHDATA);
	*s = values;
	*n = 1;
	
	return(0);
}

int
UxValuesOfXrtGphDataStyle(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_GPHDATASTYLE);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtGphDataStyles(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_GPHDATASTYLES);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtGphMarkerMethod(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphMarkerMethod;
	*n = XtNumber(uXrtGphMarkerMethod);
	return( *n);
}

int
UxValuesOfXrtGphPieOrder(s, n)
char	***s;
int		*n;
{
	*s = uXrtGphPieOrder;
	*n = XtNumber(uXrtGphPieOrder);
	return(*n);
}

int
UxValuesOfXrtGphPieThrMethod(s, n)
char	***s;
int		*n;
{
	*s = uXrtGphPieThresholdMethod;
	*n = XtNumber(uXrtGphPieThresholdMethod);
	return(*n);
}

int
UxValuesOfXrtGphRotate(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphRotate;
	*n = XtNumber(uXrtGphRotate);
	return( *n);
}

int
UxValuesOfXrtGphShading(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphShading;
	*n = XtNumber(uXrtGphShading);
	return( *n);
}

int
UxValuesOfXrtGphStrings(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_GPHSTRINGS);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtGphTimeUnit(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphTimeUnit;
	*n = XtNumber(uXrtGphTimeUnit);
	return( *n);
}

int
UxValuesOfXrtGphType(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphType;
	*n = XtNumber(uXrtGphType);
	return( *n);
}

int
UxValuesOfXrtGphValueLabels(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_GPHVALUELABELS);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtGphAnnotationMethod(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphAnnotationMethod;
	*n = XtNumber(uXrtGphAnnotationMethod);
	return( *n);
}

int
UxValuesOfXrtGphAnnoPlacement(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphAnnoPlacement;
	*n = XtNumber(uXrtGphAnnoPlacement);
	return( *n);
}

int
UxValuesOfXrtGphOriginPlacement(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphOriginPlacement;
	*n = XtNumber(uXrtGphOriginPlacement);
	return( *n);
}

int
UxValuesOfXrtGphNumMethod(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtGphNumMethod;
	*n = XtNumber(uXrtGphNumMethod);
	return( *n);
}

/*
 * Validate functions for both enumerated types and other Xtypes.
 * Use a general function for enumerated types.
 */

static int
ValidateXrtGphEnumType(rs, s, string_list, n)
swidget rs;
char	*s;
char	**string_list;
int		n;
{
	int i;

	if (s == NULL) {
		return ERROR;
	}
	for (i = 0; i < n; i++) {
		if (UxStrEqual(s, string_list[i])) {
			return NO_ERROR;
		}
	}
	return ERROR;
}

int
UxValidateXrtGphAdjust(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphAdjust,
								  XtNumber(uXrtGphAdjust)));
}

int
UxValidateXrtGphAlign(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphAlign,
								  XtNumber(uXrtGphAlign)));
}

int
UxValidateXrtGphAnchor(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphAnchor,
								  XtNumber(uXrtGphAnchor)));
}

int
UxValidateXrtGphAngleUnit(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphAngleUnit,
								  XtNumber(uXrtGphAngleUnit)));
}

int
UxValidateXrtGphBorder(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphBorder,
								  XtNumber(uXrtGphBorder)));
}

int
UxValidateXrtGphData(swgt, value)
swidget	swgt;
char	*value;
{
	XrtDataHandle data;
	char		errbuf[100];
	int			status = ERROR;

	if (!UxStrEqual(value,"")) {
		data = XrtDataCreateFromFile(value, errbuf);
		if (data) {
			status = NO_ERROR;
			XrtDataDestroy(data, True);
		}
	}
	else {
		status = NO_ERROR;
	}

	return (status);
}

int
UxValidateXrtGphMarkerMethod(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphMarkerMethod,
								  XtNumber(uXrtGphMarkerMethod)));
}

int
UxValidateXrtGphPieOrder(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphPieOrder,
								  XtNumber(uXrtGphPieOrder)));
}

int
UxValidateXrtGphPieThrMethod(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphPieThresholdMethod,
								  XtNumber(uXrtGphPieThresholdMethod)));
}

int
UxValidateXrtGphRotate(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphRotate,
								  XtNumber(uXrtGphRotate)));
}

int
UxValidateXrtGphShading(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphShading,
								  XtNumber(uXrtGphShading)));
}

int
UxValidateXrtGphTimeUnit(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphTimeUnit,
								  XtNumber(uXrtGphTimeUnit)));
}

int
UxValidateXrtGphType(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphType,
								  XtNumber(uXrtGphType)));
}

int
UxValidateXrtGphAnnotationMethod(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphAnnotationMethod,
								  XtNumber(uXrtGphAnnotationMethod)));
}

int
UxValidateXrtGphAnnoPlacement(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphAnnoPlacement,
								  XtNumber(uXrtGphAnnoPlacement)));
}

int
UxValidateXrtGphOriginPlacement(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphOriginPlacement,
								  XtNumber(uXrtGphOriginPlacement)));
}

int
UxValidateXrtGphNumMethod(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphNumMethod,
								  XtNumber(uXrtGphNumMethod)));
}

int
UxValuesOfXrtGphBubbleMethod(ulist, n)
char ***ulist;
int	*n;
{
	*ulist = uXrtGphBubbleMethod;
	*n = XtNumber(uXrtGphBubbleMethod);
	return(*n);
}

int
UxValuesOfXrtGphImage(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0]=CGETS_KL(VAOF_GPHIMAGE);
	*s = values;
	*n = 1;
	return 0;
}


int
UxValuesOfXrtGphImageLayout(ulist, n)
char ***ulist;
int	*n;
{
	*ulist = uXrtGphImageLayout;
	*n = XtNumber(uXrtGphImageLayout);
	return(*n);
}


int
UxValidateXrtGphBubbleMethod(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphBubbleMethod,
								  XtNumber(uXrtGphBubbleMethod)));
}

int
UxValidateXrtGphImageLayout(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGphEnumType(rs, s, uXrtGphImageLayout,
								  XtNumber(uXrtGphImageLayout)));
}

#endif   /* ! RUNTIME */

/*
 *	Conversion functions TO and FROM UIMX
 */

static int
File_Data(swgt, file, data, flag, xtype)
swidget			swgt;
char			**file;
XrtDataHandle	*data;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;
	Boolean         stat;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {

		info = FindXrtStringInfo(File_Data, xtype, swgt);
		if (info) {
			*file = XtNewString(info->string);
		}
		else {
			*file = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {
		
		if (**file) {
		
			from.size = strlen(*file) + 1;
			from.addr = *file;

			to.size = sizeof(XrtDataHandle);
			to.addr = (caddr_t) data;

			stat = XtConvertAndStore (UxTopLevel,
									  XtRString, &from,
									  XtRXrtData, &to);
		
			if (stat) {
				RegisterXrtStringInfo(File_Data, xtype, swgt, *file);
				status = NO_ERROR;
			}
		}
		else {
			*data = NULL;
			RegisterXrtStringInfo(File_Data, xtype, swgt, "");
			status = NO_ERROR;
		}
	}

	return(status);
}

static int
File_Data2(swgt, file, data, flag, xtype)
swidget			swgt;
char			**file;
XrtDataHandle	*data;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;
	Boolean         stat;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		info = FindXrtStringInfo(File_Data2, xtype, swgt);
		if (info) {
			*file = XtNewString(info->string);
		}
		else {
			*file = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {
		
		if (**file) {
		
			from.size = strlen( *file ) + 1;
			from.addr = *file;

			to.size = sizeof(XrtDataHandle);
			to.addr = (caddr_t) data;

			stat = XtConvertAndStore (UxTopLevel,
									  XtRString, &from,
									  XtRXrtData, &to);
		
			if (stat) {
				RegisterXrtStringInfo(File_Data2, xtype, swgt, *file);
				status = NO_ERROR;
			}
		}
		else {
			*data = NULL;
			RegisterXrtStringInfo(File_Data2, xtype, swgt, "");
			status = NO_ERROR;
		}
	}

	return(status);
}

static int
String_DataStyle(swgt, string, dstyle, flag, xtype)
swidget			swgt;
char			**string;
XrtDataStyle	**dstyle;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {

		from.size = sizeof(XrtDataStyle *);
		from.addr = (caddr_t) dstyle;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XtRXrtDataStyle, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn a data style string into a data style structure
		 */

		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(XrtDataStyle *);
		to.addr = (caddr_t) dstyle;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XtRXrtDataStyle, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

static int
String_DataStyles(swgt, string, dstyles, flag, xtype)
swidget			swgt;
char			**string;
XrtDataStyle	***dstyles;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {

		from.size = sizeof(XrtDataStyle **);
		from.addr = (caddr_t) dstyles;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XtRXrtDataStyles, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn a data styles string into a data styles structure
		 */

		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(XrtDataStyle **);
		to.addr = (caddr_t) dstyles;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XtRXrtDataStyles, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

static int
XrtGph_String_Strings(swgt, string, strings, flag, xtype)
swidget			swgt;
char			**string;
char			***strings;
int				flag;
int				xtype;
{
	int				i;
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		from.size = sizeof(char **);
		from.addr = (caddr_t) strings;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XtRXrtStrings, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn "( \"this\" \"is\" \"a\" \"list\" \"of\" \"strings\" ...)"
		 *  into something interesting.
		 */

		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(char **);
		to.addr = (caddr_t) strings;

		if (XtConvertAndStore (UxTopLevel,
							   XtRString, &from,
							   XtRXrtStrings, &to))
		{
			if (*strings) {
				for (i = 0; (*strings)[i]; i++) {
				}
			}

			status = NO_ERROR;
		}
	}
		
done:
	return(status);
}

static int
String_ValueLabels(swgt, string, labels, flag, xtype)
swidget			swgt;
char			**string;
XrtValueLabel	***labels;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		from.size = sizeof(XrtValueLabel **);
		from.addr = (caddr_t) labels;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XtRXrtValueLabels, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn a value label string into a list of value label structures
		 */

		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(XrtValueLabel **);
		to.addr = (caddr_t) labels;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XtRXrtValueLabels, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}


static int
String_Image(swgt, string, image, flag, xtype)
swidget			swgt;
char			**string;
XrtImage		*image;
int				flag;
int				xtype;
{
	int			i;
	int			status;
	XrmValue	from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  Turn a Image into a string.
		 */

		from.size = sizeof(XrtImage);
		from.addr = (caddr_t) image;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XtRXrtImage, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn string into a Image.
		 */
		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(XrtImage);
		to.addr = (caddr_t) image;

		if (XtConvertAndStore (UxTopLevel,
							   XtRString, &from,
							   XtRXrtImage, &to))
		{
			status = NO_ERROR;
		}
	}

	return(status);
}

void	
XrtGph_enum()
{
	UxXT_XrtGphAdjust = UxAddEnumType(XtRXrtAdjust, sizeof(int),
									  xXrtGphAdjust,
									  uXrtGphAdjust,
									  dXrtGphAdjust,
									  XtNumber(xXrtGphAdjust),
									  UxStringToIntEnum);
	UxXT_XrtGphAlign = UxAddEnumType(XtRXrtAlign, sizeof(int),
									 xXrtGphAlign,
									 uXrtGphAlign,
									 dXrtGphAlign,
									 XtNumber(xXrtGphAlign),
									 UxStringToIntEnum);
	UxXT_XrtGphAnchor = UxAddEnumType(XtRXrtAnchor, sizeof(int),
									  xXrtGphAnchor,
									  uXrtGphAnchor,
									  dXrtGphAnchor,
									  XtNumber(xXrtGphAnchor),
									  UxStringToIntEnum);
	UxXT_XrtGphAngleUnit = UxAddEnumType(XtRXrtAngleUnit, sizeof(int),
									  xXrtGphAngleUnit,
									  uXrtGphAngleUnit,
									  dXrtGphAngleUnit,
									  XtNumber(xXrtGphAngleUnit),
									  UxStringToIntEnum);
	UxXT_XrtGphBorder = UxAddEnumType(XtRXrtBorder, sizeof(int),
									  xXrtGphBorder,
									  uXrtGphBorder,
									  dXrtGphBorder,
									  XtNumber(xXrtGphBorder),
									  UxStringToIntEnum);
	UxXT_XrtGphMarkerMethod = UxAddEnumType(XtRXrtMarkerMethod, sizeof(int),
									  xXrtGphMarkerMethod,
									  uXrtGphMarkerMethod,
									  dXrtGphMarkerMethod,
									  XtNumber(xXrtGphMarkerMethod),
									  UxStringToIntEnum);
	UxXT_XrtGphPieOrder = UxAddEnumType(XtRXrtPieOrder, sizeof(int),
										xXrtGphPieOrder,
										uXrtGphPieOrder,
										dXrtGphPieOrder, 
										XtNumber(xXrtGphPieOrder),
										UxStringToIntEnum);
	UxXT_XrtGphPieThresholdMethod =
		UxAddEnumType(XtRXrtPieThresholdMethod, sizeof(int),
					  xXrtGphPieThresholdMethod,
					  uXrtGphPieThresholdMethod,
					  dXrtGphPieThresholdMethod,
					  XtNumber(xXrtGphPieThresholdMethod),
					  UxStringToIntEnum);
	UxXT_XrtGphRotate = UxAddEnumType(XtRXrtRotate, sizeof(int),
									  xXrtGphRotate,
									  uXrtGphRotate,
									  dXrtGphRotate,
									  XtNumber(xXrtGphRotate),
									  UxStringToIntEnum);
	UxXT_XrtGphShading = UxAddEnumType(XtRXrtShading, sizeof(int),
									  xXrtGphShading,
									  uXrtGphShading,
									  dXrtGphShading,
									  XtNumber(xXrtGphShading),
									  UxStringToIntEnum);
	UxXT_XrtGphTimeUnit = UxAddEnumType(XtRXrtTimeUnit, sizeof(int),
										xXrtGphTimeUnit,
										uXrtGphTimeUnit,
										dXrtGphTimeUnit,
										XtNumber(xXrtGphTimeUnit),
										UxStringToIntEnum);
	UxXT_XrtGphType = UxAddEnumType(XtRXrtType, sizeof(int),
									xXrtGphType,
									uXrtGphType,
									dXrtGphType,
									XtNumber(xXrtGphType),
									UxStringToIntEnum);
	UxXT_XrtGphAnnotationMethod =
		UxAddEnumType(XtRXrtAnnotationMethod, sizeof(int),
					  xXrtGphAnnotationMethod,
					  uXrtGphAnnotationMethod,
					  dXrtGphAnnotationMethod,
					  XtNumber(xXrtGphAnnotationMethod),
					  UxStringToIntEnum);
	UxXT_XrtGphAnnoPlacement = UxAddEnumType(XtRXrtAnnoPlacement, sizeof(int),
											 xXrtGphAnnoPlacement,
											 uXrtGphAnnoPlacement,
											 dXrtGphAnnoPlacement,
											 XtNumber(xXrtGphAnnoPlacement),
											 UxStringToIntEnum);
	UxXT_XrtGphOriginPlacement =
		UxAddEnumType(XtRXrtOriginPlacement, sizeof(int),
					  xXrtGphOriginPlacement,
					  uXrtGphOriginPlacement,
					  dXrtGphOriginPlacement,
					  XtNumber(xXrtGphOriginPlacement),
					  UxStringToIntEnum);
	UxXT_XrtGphNumMethod = UxAddEnumType(XtRXrtNumMethod, sizeof(int),
										 xXrtGphNumMethod,
										 uXrtGphNumMethod,
										 dXrtGphNumMethod,
										 XtNumber(xXrtGphNumMethod),
										 UxStringToIntEnum);
	
	UxXT_XrtGphBubbleMethod = UxAddEnumType(XtRXrtBubbleMethod, 
									sizeof(int),
									xXrtGphBubbleMethod,
									uXrtGphBubbleMethod,
									dXrtGphBubbleMethod,
									XtNumber(xXrtGphBubbleMethod),
									UxStringToIntEnum);
	UxXT_XrtGphImageLayout = UxAddEnumType(XtRXrtImageLayout, 
									sizeof(int),
									xXrtGphImageLayout,
									uXrtGphImageLayout,
									dXrtGphImageLayout,
									XtNumber(xXrtGphImageLayout),
									UxStringToIntEnum);

}

/* Add new XRT/graph xtypes */

void	
XrtGph_xtypes()
{
	/*
	 * Initialize widget class inorder to initialize converters
	 */

	XtInitializeWidgetClass(xtXrtGraphWidgetClass);
	XrtSetReverseConverters();

	/* Add new Xtypes */

	UxXT_float      	   = UxAddXtype(XtRFloat, sizeof(float));
	UxXT_XrtGphData        = UxAddXtype(XtRXrtData, sizeof(XrtDataHandle));
	UxXT_XrtGphData2       = UxAddXtype(XtRXrtData, sizeof(XrtDataHandle));
	UxXT_XrtGphDataStyle   = UxAddXtype(XtRXrtDataStyle,
										sizeof(XrtDataStyle *));
	UxXT_XrtGphDataStyles  = UxAddXtype(XtRXrtDataStyles,
										sizeof(XrtDataStyle **));
	UxXT_XrtGphStrings     = UxAddXtype(XtRXrtStrings, sizeof(char **));
	UxXT_XrtGphValueLabels = UxAddXtype(XtRXrtValueLabels,
										sizeof(XrtValueLabel **));
	UxXT_XrtGphImage		= UxAddXtype(XtRXrtImage, sizeof(XrtImage));

	/* Add new converters for Xtypes */
	
	UxAddConv(UxUT_string, UxXT_XrtGphData,        File_Data);
	UxAddConv(UxUT_string, UxXT_XrtGphData2,       File_Data2);
	UxAddConv(UxUT_string, UxXT_XrtGphDataStyle,   String_DataStyle);
	UxAddConv(UxUT_string, UxXT_XrtGphDataStyles,  String_DataStyles);
	UxAddConv(UxUT_string, UxXT_XrtGphStrings,     XrtGph_String_Strings);
	UxAddConv(UxUT_string, UxXT_XrtGphValueLabels, String_ValueLabels);
  	UxAddConv(UxUT_string, UxXT_XrtGphImage,	   String_Image);
	
}
