/*---------------------------------------------------------------------
 * $Source: /rcs/rcs2/include/resource.h,v $
 * $Date: 1997/06/04 21:53:11 $             $Revision: 2.10.65.2 $
 *---------------------------------------------------------------------
 * 
 *
 *             Copyright (c) 1990, Visual Edge Software Ltd.
 *
 * ALL  RIGHTS  RESERVED.  Permission  to  use,  copy,  modify,  and
 * distribute  this  software  and its documentation for any purpose
 * and  without  fee  is  hereby  granted,  provided  that the above
 * copyright  notice  appear  in  all  copies  and  that  both  that
 * copyright  notice and this permission notice appear in supporting
 * documentation,  and that  the name of Visual Edge Software not be
 * used  in advertising  or publicity  pertaining to distribution of
 * the software without specific, written prior permission. The year
 * included in the notice is the year of the creation of the work.
 *-------------------------------------------------------------------*/

/*------------------------------------------------------------------------
 * Resource_t
 *
 * Resources are the UIM/X definitions of Widget properties.
 * Each swidget class holds a list of Resources, 
 * each entry being a Resource_t created at class initialization.
 * These Resources drive the UxGet/UxPut facilities for resources,
 * and all design-time editing of initial values.
 *-----------------------------------------------------------------------*/

#ifndef RESOURCE_H
#define RESOURCE_H

#include <stdio.h>
#include <X11/Intrinsic.h>
#include "uxproto.h"
#include "plist.h"

/*------------------------------------------------------------------------
 * Types for fields.
 *------------------------------------------------------------------------*/

typedef	union	{
	float	f;
	int	i;
} FloatInt_t;

typedef int (*PutFunction_t) ();
typedef int (*GetFunction_t) ();
typedef int (*Validator_t) ();
typedef int (*ValuesOf_t) ();
typedef void (*FreeFunction_t) ();
typedef XtArgVal (*CopyFunction_t) ();

typedef enum {
	UxPASS0  = 0,		/* Widget creation argument list.	    */
	UxPASS1  = 1,		/* Design time: creation arg list.	    */
	UxPASS2  = 2		/* XtSetValues after interface tree exists. */
} Pass_t;

typedef enum {			/* The meaning of these is of overloaded.  */
	UxCORE = 0,		/* So far, the code uses this to recognize */
	UxSPECIFIC = 1,		/* constraints and callbacks all over,	   */
	UxBEHAVIOR = 2,		/* and to organize the property editor.    */
	UxCONSTRAINT = 3,
	UxDECL = 4,		/* Widget declaration properties   */
	UxCOMPOUND = 5		/* UIM/X internal compound props   */
} Division_t;


typedef enum {			/** How does the interpreter see it? **/
	UxLITERAL,		/** For name, other variable names   **/
	UxEXPRESSION,		/** For resources 		     **/
	UxSTATEMENT		/** For callbacks		     **/
} ExprTreatment_t;

/*------------------------------------------------------------------------
 * The Resource structure itself. 
 *------------------------------------------------------------------------*/

typedef struct UxResourceStruct {

	char*	Format;			/* For making C code from values;
					 * for Literals, this is a function.
					 */

	char*   UxXtName;		/* "allowShellResize" for Xt	    */
	char*   PrintName;		/* "AllowShellResize" visual labels */
	char*	UimxName;		/* "allowShellResize" as in UxPut.. */

	Division_t	Division;	/* What sort of resource is this? */
	Boolean 	Toolkit;	/* Is this an XtSetValues resource ? */
	Pass_t 		Pass;		/* 0,1,2 - which creation arglist? */
	Boolean		ReadOnly;	/* controls property table sens. */
	Boolean		CanBePublic;	/* can be written to resource file */
	short		Editable;	/* Appears in prop table ? */
	Boolean		NoEditBtn;	/* should "..." be used. */

	short 	UType;			/* uimx type (for UxPut & UxGet) */
	short 	XType;			/* type for XtSetValues   */

	Validator_t	Validator;
	ValuesOf_t	ValuesOf;
	PutFunction_t	PutFunction;
	GetFunction_t	GetFunction;

	/* These default to standard UIM/X behaviour.
	 */

	CopyFunction_t CopyFunction;	/* How to save a uimx-type value */
	FreeFunction_t FreeFunction;	/* How to free a uimx-type value */

	void       (*EditFunction)();	/* Special Purpose Editor popup  */

	ExprTreatment_t   Treatment;	/* Usually 'UxEXPRESSION'	 */
	
	void	   (*SaveFunction)();	/* When custom saving is needed. */

	XtPointer	UserDataRD;	/* For extending and customizing;
					 * the core system never touches this.
					 */
} Resource_t;

/*------------------------------------------------------------------------
 * Field access macros.
 *------------------------------------------------------------------------*/

#define UxGetDivision(res)	((res)->Division)
#define UxGetPass(res) 		((res)->Pass)

#define UxGetXtName(res) 	((res)->UxXtName) /* for backward compatability */
#define UxGetUxXtName(res) 	((res)->UxXtName)
#define UxGetPrintName(res) 	((res)->PrintName)
#define UxGetUimxName(res) 	((res)->UimxName)

#define UxGetUType(res) 	((int) ((res)->UType))
#define UxGetXType(res) 	((int) ((res)->XType))
#define UxGetValidator(res) 	((res)->Validator)
#define UxGetValuesOf(res) 	((res)->ValuesOf)
#define UxGetPutFunction(res) 	((res)->PutFunction)
#define UxGetGetFunction(res) 	((res)->GetFunction)

#define UxGetEditFunction(res) 	((res)->EditFunction)
#define UxGetFreeFunction(res) 	((res)->FreeFunction)
#define UxGetCopyFunction(res) 	((res)->CopyFunction)
#define UxGetCodeFormat(res)	((res)->Format)
#define UxGetTreatment(res)	((res)->Treatment)

#define UxGetUserDataRD(res)	((res)->UserDataRD)

/** Note that we drop the 'Get' for Booleans **/

#define UxReadOnly(res) 	((res)->ReadOnly)
#define UxCanBePublic(res) 	((res)->CanBePublic)
#define UxEditable(res) 	((res)->Editable)
#define UxNoEditBtn(res) 	((res)->NoEditBtn)
#define UxToolkit(res)		((res)->Toolkit)

/* Convenience macros **/

#define UxIsResource(res)	((res)->Division != UxBEHAVIOR && \
				 (res)->Division != UxDECL)
#define UxIsCallback(res)	((res)->Division == UxBEHAVIOR && \
				 (res)->Treatment == UxSTATEMENT)
#define UxIsCmpndCmpntRes(res)	((res)->Division == UxSPECIFIC && \
				 (res)->Toolkit == 0 && \
				 (res)->Pass == UxPASS2 && \
				 (res)->SaveFunction == UxNoopSave && \
				 (res)->UserDataRD != NULL)
#define UxIsCmpndCmpntCallback(res)	(UxIsCmpndCmpntRes(res) && \
				 	 (res)->Treatment == UxSTATEMENT)
#define UxIsLiteral(res)	((res)->Treatment == UxLITERAL)
#define UxHasValues(res)	((res)->Treatment != UxSTATEMENT)



#include "ival.h"	/* to get the def'n of SourceFlag_t */

/*------------------------------------------------------------------------
 * Methods
 *------------------------------------------------------------------------*/

extern vhandle      UxParseResource UXPROTO(( swidget, Resource_t *, char * ));
extern vhandle      UxParseProperty UXPROTO(( swidget, Resource_t *, char *,
						SourceFlag_t ));

/*------------------------------------------------------------------------
 * Constructor.
 * UxDefineResource supplies defaults for many fields; see resource.c.
 * All calls to UxDefineResource appear in swidget class initializers.
 *------------------------------------------------------------------------*/

typedef enum {
	RD_END,			/* ALWAYS the last arg (ends vararg list) */
	RD_NAME,		/* a string -- triggers defaults for 3 names */
	RD_UTYPE,		/* a UxUT_ */
	RD_XTYPE,		/* a UxXT_ */
	RD_XTNAME,		/* string for XtSetValues */
	RD_PRINTNAME,		/* string for user */
	RD_UIMXNAME,		/* string for iface files */
	RD_VALIDATOR,		/* function */
	RD_VALUESOF,		/* function */
	RD_PUTFUNCTION,		/* function */
	RD_GETFUNCTION,		/* function */
	RD_TOOLKIT,		/* TRUE/FALSE - passed in XtSetValues? */
	RD_PASS,		/* 0, 1, 2 */
	RD_DIVISION,		/* Division_t enum */
	RD_READONLY,		/* TRUE/FALSE - prop tab entry editable ? */
	RD_CANBEPUBLIC,		/* TRUE/FALSE */
	RD_EDITABLE,		/* TRUE/FALSE - appears in prop tab? */
	RD_NOEDITBTN,		/* TRUE/FALSE - "..." appears in PE row? */
	RD_COPYFUNCTION,	/* function */
	RD_FREEFUNCTION,	/* function */
	RD_EDITFUNCTION,	/* function */
	RD_EXAMPLE,		/* special - value is a Resource* to copy. */
	RD_FORMAT,		/* string, default "\"%s\"" */
	RD_TREATMENT,		/* ExprTreatment_t, default EXPRESSION */
	RD_SAVEFUNCTION,	/* function, when custom saving is needed. */
	RD_USERDATARD		/* allows user to add own data. */
} RD_Key_t;

extern Resource_t * UxDefineResource UXPROTO((RD_Key_t,...));

/*------------------------------------------------------------------------
 * Access.  For extensibility, this lists ALL resources defined.
 *------------------------------------------------------------------------*/

extern PList_c * UxGetAllResources UXPROTO(( void ));
extern void	 UxGlobalInstanceResource UXPROTO((char *argument,
						   Resource_t *res));
extern void	 UxInstanceResource UXPROTO((char *component,
					     char *argument,
					     Resource_t *res));

/*------------------------------------------------------------------------
 * Building blocks for extending save of properties.
 *	BasicWriteProperty -- handles value, and lock and source facets.
 *------------------------------------------------------------------------*/

void   UxBasicWriteProperty 
		UXPROTO((swidget, IVal_t*, char* objname, FILE* fp));

/*------------------------------------------------------------------------
 * Building blocks for extending save of properties.
 *	WriteFacets -- handles lock and source facets.
 *------------------------------------------------------------------------*/

void   UxWriteFacets   
		UXPROTO((swidget, IVal_t*, char* objname, FILE* fp));

/*------------------------------------------------------------------------
 *  The master function for writing a property+facets to a stream.
 *------------------------------------------------------------------------*/

void   UxWriteProperty 
		UXPROTO((swidget, IVal_t*, char* objname, FILE* fp));

#endif /* RESOURCE_H */

